//-------------------------------------------------------------------------
// Titel        : "Digitales Raumthermostat"
//-------------------------------------------------------------------------
// Funktion     : Digitales Raumthermostat mit Grafik-Display, Real-Time- 
//              : Clock, Bewegungsmelder fr Temperatursteuerung und 
//				: Digitalen Temperatursensor DS18S20
//-------------------------------------------------------------------------
// Prozessor    : ATmega169V
// Takt         : 1.0 MHz / 3.3 Volt 
// Sprache      : C
// C-Compiler   : WinAVR - 20080610 (programmiert in AVR-Studio Vers. 4.18) 
//              : (http://sourceforge.net/projects/winavr/)
// Datum        : 23.04.2010
// Version      : 1.0
// Autor        : Rolf Degen
// eMail		: rolfdegen@hotmail.com
//-------------------------------------------------------------------------

#define     DF_CPU 1000000         	// Taktfrequenz ATmega
#include    <avr\io.h>             	// AVR Register und Konstantendefinitionen
#include    <util\delay.h>         	// Bibliothek mit Warteroutinen (es wird kein Timer im ATmega verwendet)
#include    <avr\interrupt.h>      	// Timer-Interuppts
#include 	<avr/pgmspace.h>		// Zugriff auf int. Programmspeicher
#include	<avr/eeprom.h>			// Zugriff auf int. EEPROM

//-- Funktions-Deklarationen ----------------------
void dogSPIout(char out);
void initDOGM128();
void ResetDOG(void);
void DogKontrast(void);
void clr_disp();
void set_disp_pos(char x_pos, char y_pos);
void print_char(uint8_t asc_data);
void print_big_ziffer(char xpos, char data);
void print_symbol(const char *addr, uint8_t xpos, uint8_t ypos);
void print_thermometer();
void init_rtc();
void read_time();
void print_realtime();
void print_zeit();
void print_datum();
void print_wochentag();
void read_keys();
void set_regler_off();
void set_regler_auto();
void temperatur_anzeigen();
void ds1820_start();
void ds1820_reset();
void ds1820_write_bit();
void ds1820_write_byte();
void ds1820_read_bit();
void ds1820_read_byte();
void ds1820_read_temp();
void ds1820_print_temp();
void inc_temp_soll();
void dec_temp_soll();
void print_temperatur();
void print_text(const char *addr);	// Textausgabe
void print_grundanzeige();			// Men "Grundanzeige"
void bewegungsmelder();
void komfort_funktion();
void cursor_flash();
void print_zeit();
void print_realdate();
void write_eeprom();
void read_eeprom();
void write_rtc_date();
void print_digit(uint8_t digit);
void lcd_helligkeit();
void print_digit_16(uint16_t zahl);
void vergleich();
void print_big_temperatur();
void flashing_big_ziffer();
void temperatur_anzeigen();
void system_reset();

// ---- Portleitungen Temperatursensor
#define sens_port     PORTE         	// Sensor-Port
#define sens_ddr      DDRE          	// Port-Richtung Temperatursensor
#define sens_inp      PINE          	// Input-Leitung Temperatursensor
#define sensor1       PE1           	// DS18S20 Temperatursensor1

// ---- Portleitung fr LCD-Beleuchtung
#define pwm_light     PB4           	// Portpin PWM-Signal LCD-Beleuchtung


// ---- Variablen fr Temperaturmessung
volatile uint8_t 	ds1820_bit = 0;         // Sende- u. Empfangs-Bit  Temperatursensor
volatile uint8_t 	ds1820_byte = 0;        // Sende- u. Empfangs-Byte Temperatursensor
volatile uint8_t    ds1820_lsb=0;			// Low-Byte  Temperaturmesswert
volatile uint8_t	ds1820_msb=0;			// High-Byte Temperaturmesswert
volatile uint8_t    ds1820_countremain=0;	// Coutremain-Wert  Temperatursensor 
volatile uint8_t    ds1820_countperc=0;		// Countpersec-Wert Temperatursensor
volatile uint16_t   ds1820_temperatur=0;	// Temperaturwert 16Bit
volatile uint8_t 	ds1820_messzyklus = 0;	// Temperaturmesszyklus (alle 7 sec)
volatile uint8_t    ds1820_stop_flag=0;		// Flag=1 keine Temperaturmessung
volatile uint8_t    ds1820_busy_flag=0;		// 0=nicht aktiv 1=Messung luft  2=Messung beendet
volatile uint8_t    ds1820_err_flag=0;		// Flag=1 wenn Fehler nach Reset-Signal an Sensor
volatile uint8_t 	ds1820_minus = 0;	    // ds1820_minus=1 negative Temperatur
volatile uint16_t   temperatur=0;
volatile uint8_t	regler=0;				// 0=Temperatur-Regler off, 1=autmatisch, 2=manuell
volatile uint8_t	heizen_flag=0;			// Flag=1 wenn Relais an
volatile uint16_t	temp_soll=0;			// Temperatur-Sollwert
volatile uint16_t	temp_soll1=0;			// neuer Temperatur-Sollwert
volatile uint16_t   temp_soll_eeprom;	    // gespeicherter_Temperatur-Sollwert
volatile uint16_t	temp_komfort=0;			// Temperatur-Komfortwert
volatile uint16_t   temp_nacht=0;			// Temperatur Nachtabsenkung
volatile uint8_t	pir_flag=0;				// Flag=1 Signal vom Bewegungsmelder
volatile uint8_t	pir_timer=0;			// Zeit in sec fr Komfort Symbol-Anzeige
volatile uint8_t	pir_timer_on=0;			// Flag=1 wenn pir_timer luft
volatile uint8_t	komfort_flag=0;			// Flag=1 Komforttemperatur aktiviert
volatile uint8_t	komfort_temp_funktion=0;		// Komfort-Funktion 0=aus 1=ein
volatile uint16_t	komfort_timer=0;		// Zeit in sec fr Komfort-Temperatur Laufzeit
volatile uint8_t	komfort_timer_on=0;		// Flag=1 wenn komfort_timer luft
volatile uint8_t	face_timer=0;
volatile uint8_t	face_timer_on=0;
volatile uint8_t	menue_timer_on=0;		// Men-Timer beendet die Meneingabe automatisch nach 5min
volatile uint16_t	menue_timer=0;			// Men-Timer-Wert
volatile uint8_t	cursor_on=0;			// Cursor-Flag   1=Cursor eingeschaltet 0=Cursor ausgeschaltet
volatile uint8_t	cursor_flashing=0;		// Cursor-Symbol 1=Symbol sichtbar 0=Symbol unsichtbar
volatile uint8_t	cursor_ypos=0;			// Cursor-Position senkrecht  (0-8 Display Spalten)
volatile uint8_t	cursor_xpos=0;			// Cursor-Position waagerecht (0-127 Display Pixel-Breite)
volatile uint8_t	cursor_timer=0;
volatile uint8_t	change_flag=0;			// Flag=1 bei nderung der Men-Werten
volatile uint8_t    change_time_flag=0;		// Flag=1 bei nderung der Uhrzeit
volatile uint8_t	change_timedigit=0;		// 0=Stunden stellen  1=Minuten stellen  2=Sekunden stellen 
volatile uint8_t    flash_big_ziffer=0;		// Blinkende Temperaturanzeige
volatile uint8_t	flash_big_ziffer_timer=0; // Timer fes blinken
volatile uint8_t	int_counter=0;			// Zhlt Interuppt-Durchlufe von Timer1

// ---- Speicherbereich im EEPROM
uint16_t eetemp_soll 				EEMEM = 0x00;	// Soll-Temperatur
uint16_t eetemp_komfort 			EEMEM = 0x01;	// Komfort-Temperatur
uint16_t eetemp_nacht				EEMEM = 0x02;	// Nacht-Temperatur
uint8_t	 eekomfort_temp_funktion  	EEMEM = 0x03; 	// Komfort-Funktion
uint8_t	 eelcd_helligkeit			EEMEM = 0x04;	// Display-Helligkeit
uint8_t	 eenacht_von_std			EEMEM = 0x05;	// Nachtabsenkung von bis
uint8_t	 eenacht_von_min			EEMEM = 0x06;	// Nachtabsenkung von bis
uint8_t	 eenacht_bis_std			EEMEM = 0x07;	// Nachtabsenkung von bis
uint8_t	 eenacht_bis_min			EEMEM = 0x08;	// Nachtabsenkung von bis

// ---- Portleitung fr Relais
#define relais_port     PORTF         					// Relais-Port
#define relais_ddr      DDRF          					// Port-Richtung Relais
#define relais          PF7           					// Relais fr Heizungslfter
#define relais_on    	relais_port |= (1 << relais)	// Relais-Leitung auf high schalten
#define relais_off 		relais_port &=~(1 << relais)	// Relais-Leitung auf low  schalten
#define relais_out 		relais_ddr |=  (1 << DDF7)		// Relais-Portleitung auf Ausgang schalten

// ---- Portleitungen Tasten
#define key_port     	PORTB         	// Tasten-Port
#define key_ddr      	DDRB          	// Port-Richtung Tasten
//#define key_left        PB2           	// Linke Men-Taste
#define key_left_inp    PINB2          	// Input-Leitung Men-Taste
#define key_left_in		key_ddr &= ~(1 << DDB2)	// Tastenleitung auf Eingang schalten
volatile uint8_t		press_key=0;	// wenn eine Taste gedrckt ist
volatile uint8_t		key_left=0;		// Men-Taste "<<"
volatile uint8_t		key_right=0;		// Men-Taste ">>"
volatile uint8_t		key_plus=0;		// Men-Taste "+"
volatile uint8_t		key_minus=0;		// Men-Taste "-"
volatile uint8_t		key_left_status=0;
volatile uint8_t		key_right_status=0;
volatile uint8_t		key_plus_status=0;
volatile uint8_t		key_minus_status=0;
volatile uint8_t		menue_site=0;		// 0=Hauptseite 1=Temperaturen 2=Nachtabschaltung 3=Grunddaten
volatile uint16_t		text_nr=0;
volatile uint8_t		menue_funktion=0;	// 0=Menseitenauswahl 1=Men-Werte ndern
volatile uint8_t		menue_auswahl=0;	// Funktionsauswahl im Men 1- max 6

// ---- Portleitung fr I2C-Bus
#define scl 			PE4		// SCL Port-Pin am ATmega
#define sda 			PE5		// SDA Port-Pin am ATmega
#define i2c_port 		PORTE	// Ausgabe-Port E am ATmega
#define i2c_ddr 		DDRE	// Datenrichtungs-Register E im ATmega

#define scl_inp 		PINE4	// SCL-Eingangsleitung
#define sda_inp 		PINE5	// SDA-Eingangsleitung

#define scl_high    	i2c_port |= (1 << scl)	// SCL-Leitung auf high schalten
#define scl_low 		i2c_port &=~(1 << scl)	// SCL-Leitung auf low  schalten
#define sda_high    	i2c_port |= (1 << sda)	// SDA-Leitung auf high schalten
#define sda_low 		i2c_port &=~(1 << sda)	// SDA-Leitung auf low  schalten

#define sda_out 		i2c_ddr |=  (1 << DDE5)	// SDA-Portleitung auf Ausgang schalten
#define sda_in			i2c_ddr &= ~(1 << DDE5)	// SDA-Portleitung auf Eingang schalten
#define scl_out         i2c_ddr |=  (1 << DDE4)	// SCL-Portleitung auf Ausgang schalten
#define scl_in 		 	i2c_ddr &= ~(1 << DDE4)	// SCL-Portleitung auf Eingang schalten
volatile uint8_t		i2c_data=0;				// I2C Sende- oder Empfangs-Byte 

volatile uint8_t		testbyte=0;				// I2C Sende- oder Empfangs-Byte 

volatile uint8_t		sekunde=0;	
volatile uint8_t		minute=0;	
volatile uint8_t		stunde=0;
volatile uint8_t		digit_stunde=0;			// temporrer Zwischenspeicher fr Editierung
volatile uint8_t		digit_minute=0;
volatile uint8_t		digit_sekunde=0;
volatile uint8_t		tag=0;
volatile uint8_t		wochentag=0;
volatile uint8_t		monat=0;
volatile uint8_t		jahr=0;
volatile uint8_t		nacht_von_std=0;
volatile uint8_t		nacht_von_min=0;
volatile uint8_t		nacht_bis_std=0;
volatile uint8_t		nacht_bis_min=0;
volatile uint16_t		nacht_time=0;
volatile uint16_t		nacht_von=0;
volatile uint16_t		nacht_bis=0;
volatile uint8_t		nacht_flag=0;			// Flag=1 Nachtabsenkung aktiv
volatile uint8_t		doppelpunkt=1;
volatile uint8_t		helligkeit=0;
volatile uint8_t		helligkeitsstufe=0;
volatile uint8_t	    last_adc_wert=0;
volatile uint8_t		adc_wert=0;			
volatile uint8_t		time_flag=0;
volatile uint8_t		sensor_flag=0;
volatile uint8_t		sec_count=0;


//-- Konstanten fr Display-Kontroller ST7565 -----
#define 	DISPOFF      0xAE
#define 	DISPON       0xAF
#define 	DISPSTART    0x40
#define 	PAGEADR      0xB0
#define 	COLADRL      0x00
#define 	COLADRH      0x10
#define 	ADCNORMAL    0xA0
#define 	ADCREVERSE   0xA1
#define 	COMNORMAL    0xC0
#define 	COMREVERSE   0xC8
#define 	DISPNORMAL   0xA6
#define 	DISPREVERSE  0xA7
#define 	LCDBIAS9     0xA2
#define 	LCDBIAS7     0xA3
#define 	RESET        0xE2
#define 	SETPOWERCTRL 0x28
#define 	REGRESISTOR  0x20
#define 	SETCONTRAST  0x81
//#define 	STATINDMODE  0xAC
#define 	STATINDMODE  0b10101101
#define 	BOOSTERRATIO 0xF8
#define 	all_points_on 	0b10100101
#define 	all_points_off 	0b10100100
#define 	disp_normal		0b10100110
#define 	disp_reverse	0b10100111
#define		disp_indicator_blink  0b00000010
#define 	disp_indicator_on	  0b10101101
#define 	disp_indicator_off	  0b10101100

//-- Variable -------------------------------------
volatile 	uint8_t disp_data=0;
volatile 	uint8_t page=0;
volatile 	uint8_t col_l=0;
volatile 	uint8_t col_h=0;
volatile 	uint8_t contrast=0;
volatile 	uint8_t Xpixel=0;
volatile 	uint8_t Ypages=0;
volatile 	uint8_t xoffset=0;
volatile	uint8_t inverse=0;	// inverse Textausgabe

//-- Makrodefinitionen ----------------------------
#define 	SetBit(adr, bnr)	( (adr) |=  (1 << (bnr)) )
#define 	ClrBit(adr, bnr)	( (adr) &= ~(1 << (bnr)) )
#define		DOGENABLE  ClrBit(DOGCSPORT, DOGCS)
#define		DOGDISABLE SetBit(DOGCSPORT, DOGCS)
#define		DOGCOMMAND ClrBit(DOGA0PORT, DOGA0)
#define		DOGDATA    SetBit(DOGA0PORT, DOGA0)

//-- ATmega Portdefinitionen ----------------------
#define 	DOGCSPORT	PORTA
#define 	DOGCS		0
#define 	DOGRESPORT	PORTD
#define 	DOGRES		7
#define 	DOGA0PORT	PORTA
#define 	DOGA0		1
#define 	DOGSCLPORT	PORTA
#define 	DOGSCL		2
#define 	DOGSIPORT	PORTA
#define 	DOGSI		3

//----------------------------------------------------------
//  Zeichen-Font (8*6 Pixel) 128 Zeichen
//----------------------------------------------------------
const uint8_t font[] PROGMEM =
{
0x00,0x7F,0x3E,0x1C,0x08,0x00,	// 0x00 Cursor-Symbol
0x00,0x06,0x09,0x09,0x06,0x00,	// 0x01 Grad-Symbol
0x00,0x3C,0x41,0x40,0x41,0x3C,	// 0x02 ""
0x00,0x20,0x55,0x54,0x55,0x78,	// 0x03 ""
0x00,0x38,0x45,0x44,0x45,0x38,	// 0x04 ""
0x00,0x30,0x36,0x7F,0x36,0x30,	// 0x05
0x00,0x18,0x5C,0x7E,0x5C,0x18,	// 0x06
0x00,0x00,0x00,0x00,0x00,0x00,	// 0x07
0x00,0x00,0x00,0x00,0x00,0x00,	// 0x08
0x00,0x00,0x00,0x00,0x00,0x00,	// 0x09
0x00,0x00,0x00,0x00,0x00,0x00,	// 0x0A
0x00,0x30,0x48,0x4A,0x36,0x0E,	// 0x0B
0x00,0x06,0x29,0x79,0x29,0x06,	// 0x0C
0x00,0x00,0x00,0x00,0x00,0x00,	// 0x0D
0x00,0x60,0x7E,0x0A,0x35,0x3F,	// 0x0E
0x00,0x2A,0x1C,0x36,0x1C,0x2A,	// 0x0F
0x00,0x00,0x7F,0x3E,0x1C,0x08,	// 0x10
0x00,0x08,0x1C,0x3E,0x7F,0x00,	// 0x11
0x00,0x14,0x36,0x7F,0x36,0x14,	// 0x12
0x00,0x00,0x5F,0x00,0x5F,0x00,	// 0x13
0x00,0x06,0x09,0x7F,0x01,0x7F,	// 0x14
0x00,0x22,0x4D,0x55,0x59,0x22,	// 0x15
0x00,0x60,0x60,0x60,0x60,0x00,	// 0x16
0x00,0x14,0xB6,0xFF,0xB6,0x14,	// 0x17
0x00,0x04,0x06,0x7F,0x06,0x04,	// 0x18
0x00,0x10,0x30,0x7F,0x30,0x10,	// 0x19
0x00,0x08,0x08,0x3E,0x1C,0x08,	// 0x1A
0x00,0x08,0x1C,0x3E,0x08,0x08,	// 0x1B
0x00,0x78,0x40,0x40,0x40,0x40,	// 0x1C
0x00,0x08,0x3E,0x08,0x3E,0x08,	// 0x1D
0x00,0x30,0x3C,0x3F,0x3C,0x30,	// 0x1E
0x00,0x03,0x0F,0x3F,0x0F,0x03,	// 0x1F
0x00,0x00,0x00,0x00,0x00,0x00,	// 0x20 = " "
0x00,0x00,0x06,0x5F,0x06,0x00,	// 0x21
0x00,0x07,0x03,0x00,0x07,0x03,	// 0x22
0x00,0x24,0x7E,0x24,0x7E,0x24,	// 0x23
0x00,0x24,0x2B,0x6A,0x12,0x00,	// 0x24
0x00,0x63,0x13,0x08,0x64,0x63,	// 0x25
0x00,0x36,0x49,0x56,0x20,0x50,	// 0x26
0x00,0x00,0x07,0x03,0x00,0x00,	// 0x27
0x00,0x00,0x3E,0x41,0x00,0x00,	// 0x28
0x00,0x00,0x41,0x3E,0x00,0x00,	// 0x29
0x00,0x08,0x3E,0x1C,0x3E,0x08,	// 0x2A
0x00,0x08,0x08,0x3E,0x08,0x08,	// 0x2B
0x00,0x00,0xE0,0x60,0x00,0x00,	// 0x2C
0x00,0x08,0x08,0x08,0x08,0x08,	// 0x2D
0x00,0x00,0x60,0x60,0x00,0x00,	// 0x2E
0x00,0x20,0x10,0x08,0x04,0x02,	// 0x2F
0x00,0x3E,0x51,0x49,0x45,0x3E,	// 0x30 = "0"
0x00,0x00,0x42,0x7F,0x40,0x00,	// 0x31 = "1"
0x00,0x62,0x51,0x49,0x49,0x46,	// 0x32 = "2"
0x00,0x22,0x49,0x49,0x49,0x36,	// 0x33 = "3"
0x00,0x18,0x14,0x12,0x7F,0x10,	// 0x34 = "4"
0x00,0x2F,0x49,0x49,0x49,0x31,	// 0x35 = "5"
0x00,0x3C,0x4A,0x49,0x49,0x30,	// 0x36
0x00,0x01,0x71,0x09,0x05,0x03,	// 0x37
0x00,0x36,0x49,0x49,0x49,0x36,	// 0x38
0x00,0x06,0x49,0x49,0x29,0x1E,	// 0x39
0x00,0x00,0x6C,0x6C,0x00,0x00,	// 0x3A
0x00,0x00,0xEC,0x6C,0x00,0x00,	// 0x3B
0x00,0x08,0x14,0x22,0x41,0x00,	// 0x3C
0x00,0x24,0x24,0x24,0x24,0x24,	// 0x3D
0x00,0x00,0x41,0x22,0x14,0x08,	// 0x3E
0x00,0x02,0x01,0x59,0x09,0x06,	// 0x3F
0x00,0x3E,0x41,0x5D,0x55,0x1E,	// 0x40
0x00,0x7E,0x11,0x11,0x11,0x7E,	// 0x41
0x00,0x7F,0x49,0x49,0x49,0x36,	// 0x42
0x00,0x3E,0x41,0x41,0x41,0x22,	// 0x43
0x00,0x7F,0x41,0x41,0x41,0x3E,	// 0x44
0x00,0x7F,0x49,0x49,0x49,0x41,	// 0x45
0x00,0x7F,0x09,0x09,0x09,0x01,	// 0x46
0x00,0x3E,0x41,0x49,0x49,0x7A,	// 0x47
0x00,0x7F,0x08,0x08,0x08,0x7F,	// 0x48
0x00,0x00,0x41,0x7F,0x41,0x00,	// 0x49
0x00,0x30,0x40,0x40,0x40,0x3F,	// 0x4A
0x00,0x7F,0x08,0x14,0x22,0x41,	// 0x4B
0x00,0x7F,0x40,0x40,0x40,0x40,	// 0x4C
0x00,0x7F,0x02,0x04,0x02,0x7F,	// 0x4D
0x00,0x7F,0x02,0x04,0x08,0x7F,	// 0x4E
0x00,0x3E,0x41,0x41,0x41,0x3E,	// 0x4F
0x00,0x7F,0x09,0x09,0x09,0x06,	// 0x50
0x00,0x3E,0x41,0x51,0x21,0x5E,	// 0x51
0x00,0x7F,0x09,0x09,0x19,0x66,	// 0x52
0x00,0x26,0x49,0x49,0x49,0x32,	// 0x53
0x00,0x01,0x01,0x7F,0x01,0x01,	// 0x54
0x00,0x3F,0x40,0x40,0x40,0x3F,	// 0x55
0x00,0x1F,0x20,0x40,0x20,0x1F,	// 0x56
0x00,0x3F,0x40,0x3C,0x40,0x3F,	// 0x57
0x00,0x63,0x14,0x08,0x14,0x63,	// 0x58
0x00,0x07,0x08,0x70,0x08,0x07,	// 0x59
0x00,0x71,0x49,0x45,0x43,0x00,	// 0x5A
0x00,0x00,0x7F,0x41,0x41,0x00,	// 0x5B
0x00,0x02,0x04,0x08,0x10,0x20,	// 0x5C
0x00,0x00,0x41,0x41,0x7F,0x00,	// 0x5D
0x00,0x04,0x02,0x01,0x02,0x04,	// 0x5E
0x80,0x80,0x80,0x80,0x80,0x80,	// 0x5F
0x00,0x00,0x03,0x07,0x00,0x00,	// 0x60
0x00,0x20,0x54,0x54,0x54,0x78,	// 0x61
0x00,0x7F,0x44,0x44,0x44,0x38,	// 0x62
0x00,0x38,0x44,0x44,0x44,0x28,	// 0x63
0x00,0x38,0x44,0x44,0x44,0x7F,	// 0x64
0x00,0x38,0x54,0x54,0x54,0x08,	// 0x65
0x00,0x08,0x7E,0x09,0x09,0x00,	// 0x66
0x00,0x18,0xA4,0xA4,0xA4,0x7C,	// 0x67
0x00,0x7F,0x04,0x04,0x78,0x00,	// 0x68
0x00,0x00,0x00,0x7D,0x40,0x00,	// 0x69
0x00,0x40,0x80,0x84,0x7D,0x00,	// 0x6A
0x00,0x7F,0x10,0x28,0x44,0x00,	// 0x6B
0x00,0x00,0x00,0x7F,0x40,0x00,	// 0x6C
0x00,0x7C,0x04,0x18,0x04,0x78,	// 0x6D
0x00,0x7C,0x04,0x04,0x78,0x00,	// 0x6E
0x00,0x38,0x44,0x44,0x44,0x38,	// 0x6F
0x00,0xFC,0x44,0x44,0x44,0x38,	// 0x70
0x00,0x38,0x44,0x44,0x44,0xFC,	// 0x71
0x00,0x44,0x78,0x44,0x04,0x08,	// 0x72
0x00,0x08,0x54,0x54,0x54,0x20,	// 0x73
0x00,0x04,0x3E,0x44,0x24,0x00,	// 0x74
0x00,0x3C,0x40,0x20,0x7C,0x00,	// 0x75
0x00,0x1C,0x20,0x40,0x20,0x1C,	// 0x76
0x00,0x3C,0x60,0x30,0x60,0x3C,	// 0x77
0x00,0x6C,0x10,0x10,0x6C,0x00,	// 0x78
0x00,0x9C,0xA0,0x60,0x3C,0x00,	// 0x79
0x00,0x64,0x54,0x54,0x4C,0x00,	// 0x7A
0x00,0x08,0x3E,0x41,0x41,0x00,	// 0x7B
0x00,0x00,0x00,0x77,0x00,0x00,	// 0x7C
0x00,0x00,0x41,0x41,0x3E,0x08,	// 0x7D
0x00,0x02,0x01,0x02,0x01,0x00,	// 0x7E
0x00,0x3C,0x26,0x23,0x26,0x3C,	// 0x7F
};

//----------------------------------------------------------
//  Display Bitmap-Segment-Nr. (8x9) fr Ziffern 0-9
//	Tabelle Bitmap-Sekmente fr groe Ziffern 
//----------------------------------------------------------
const uint8_t sek_tabel[] PROGMEM =
{
// Ziffer "0"
1,4,5,8,0,9,11,0,16,18,0,23,8,0,9,24,27,28,31,32,33,
// Ziffer "1"
0,0,6,0,0,9,0,0,16,0,0,23,0,0,9,0,0,30,0,0,0,
// Ziffer "2"
2,4,5,0,0,9,12,13,14,17,20,22,8,0,0,24,27,29,31,32,33, 
// Ziffer "3"
2,4,5,0,0,9,12,13,14,19,20,21,0,0,9,26,27,28,31,32,33,
// Ziffer "4"
3,0,6,8,0,9,10,13,14,19,20,21,0,0,9,0,0,30,0,0,0,
// Ziffer "5"
1,4,7,8,0,0,10,13,15,19,20,21,0,0,9,26,27,28,31,32,33,
// Ziffer "6"
1,4,7,8,0,0,10,13,15,17,20,21,8,0,9,24,27,28,31,32,33,
// Ziffer "7"
2,4,5,0,0,9,0,0,16,0,0,23,0,0,9,0,0,30,0,0,0,
// Ziffer "8"
1,4,5,8,0,9,10,13,14,17,20,21,8,0,9,24,27,28,31,32,33,
// Ziffer "9"
1,4,5,8,0,9,10,13,14,19,20,21,0,0,9,26,27,28,31,32,33,
// Minus - Symbole ( Tabellen-Index 10)
0,0,0,0,0,0,12,13,15,19,20,22,0,0,0,0,0,0,0,0,0,
// Leerzeichen ( Tabellen-Index 11)
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,

};

//-----------------------------------------------------------
//  Display Bitmap-Segment-Font (8x9) fr Ziffern 0-9
//  Groe Ziffer von 0-9 fr Temperaturanzeige
//  ---------------------------------------------------------
const uint8_t bitmap_font[] PROGMEM =
{
0b00000000,	// Bitmap 0
0b00000000,
0b00000000,
0b00000000,
0b00000000,
0b00000000,
0b00000000,
0b00000000,
0b00000000,

0b11111000,	// Bitmap 1
0b11111100,
0b11111110,
0b11111100,
0b11111001,
0b11110011,
0b00000111,
0b00001111,
0b00011111,

0b00000000,	// Bitmap 2
0b00000000,
0b00000000,
0b00000000,
0b00000001,
0b00000011,
0b00000111,
0b00001111,
0b00011111,

0b11111000,	// Bitmap 3
0b11111100,
0b11111110,
0b11111100,
0b11111000,
0b11110000,
0b00000000,
0b00000000,
0b00000000,

0b00011111,	// Bitmap 4
0b00011111,
0b00011111,
0b00011111,
0b00011111,
0b00011111,
0b00011111,
0b00011111,
0b00011111,

0b00011111,	// Bitmap 5
0b00001111,
0b00000111,
0b11110011,
0b11111001,
0b11111100,
0b11111110,
0b11111100,
0b11111000,

0b00000000,	// Bitmap 6
0b00000000,
0b00000000,
0b11110000,
0b11111000,
0b11111100,
0b11111110,
0b11111100,
0b11111000,

0b00011111,	// Bitmap 7
0b00001111,
0b00000111,
0b00000011,
0b00000001,
0b00000000,
0b00000000,
0b00000000,
0b00000000,

0b11111111,	// Bitmap 8
0b11111111,
0b11111111,
0b11111111,
0b11111111,
0b11111111,
0b00000000,
0b00000000,
0b00000000,

0b00000000,	// Bitmap 9
0b00000000,
0b00000000,
0b11111111,
0b11111111,
0b11111111,
0b11111111,
0b11111111,
0b11111111,

0b00111111,	// Bitmap 10
0b01111111,
0b11111111,
0b01111111,
0b00111111,
0b10011111,
0b11000000,
0b11000000,
0b11000000,

0b00111111,	// Bitmap 11
0b01111111,
0b11111111,
0b01111111,
0b00111111,
0b00011111,
0b00000000,
0b00000000,
0b00000000,

0b00000000,	// Bitmap 12
0b00000000,
0b00000000,
0b00000000,
0b00000000,
0b10000000,
0b11000000,
0b11000000,
0b11000000,

0b11000000, // Bitmap 13
0b11000000,
0b11000000,
0b11000000,
0b11000000,
0b11000000,
0b11000000,
0b11000000,
0b11000000,

0b11000000,	// Bitmap 14
0b11000000,
0b11000000,
0b10011111,
0b00111111,
0b01111111,
0b11111111,
0b01111111,
0b00111111,

0b11000000,	// Bitmap 15
0b11000000,
0b11000000,
0b10000000,
0b00000000,
0b00000000,
0b00000000,
0b00000000,
0b00000000,

0b00000000,	// Bitmap 16
0b00000000,
0b00000000,
0b00011111,
0b00111111,
0b01111111,
0b11111111,
0b01111111,
0b00111111,

0b11111000,	// Bitmap 17
0b11111100,
0b11111110,
0b11111100,
0b11111001,
0b11110011,
0b00000111,
0b00000111,
0b00000111,

0b11111000,	// Bitmap 18
0b11111100,
0b11111110,
0b11111100,
0b11111000,
0b11110000,
0b00000000,
0b00000000,
0b00000000,

0b00000000,	// Bitmap 19
0b00000000,
0b00000000,
0b00000000,
0b00000001,
0b00000011,
0b00000111,
0b00000111,
0b00000111,

0b00000111,	// Bitmap 20
0b00000111,
0b00000111,
0b00000111,
0b00000111,
0b00000111,
0b00000111,
0b00000111,
0b00000111,

0b00000111,	// Bitmap 21
0b00000111,
0b00000111,
0b11110011,
0b11111001,
0b11111100,
0b11111110,
0b11111100,
0b11111000,

0b00000111,	// Bitmap 22
0b00000111,
0b00000111,
0b00000011,
0b00000001,
0b00000000,
0b00000000,
0b00000000,
0b00000000,

0b00000000,	// Bitmap 23
0b00000000,
0b00000000,
0b11110000,
0b11111000,
0b11111100,
0b11111110,
0b11111100,
0b11111000,

0b00111111,	// Bitmap 24
0b01111111,
0b11111111,
0b01111111,
0b00111111,
0b10011111,
0b11000000,
0b11100000,
0b11110000,

0b00111111,	// Bitmap 25
0b01111111,
0b11111111,
0b01111111,
0b00111111,
0b00011111,
0b00000000,
0b00000000,
0b00000000,

0b00000000,	// Bitmap 26
0b00000000,
0b00000000,
0b00000000,
0b00000000,
0b10000000,
0b11000000,
0b11100000,
0b11110000,

0b11110000,	// Bitmap 27
0b11110000,
0b11110000,
0b11110000,
0b11110000,
0b11110000,
0b11110000,
0b11110000,
0b11110000,

0b11110000,	// Bitmap 28
0b11100000,
0b11000000,
0b10011111,
0b00111111,
0b01111111,
0b11111111,
0b01111111,
0b00111111,

0b11110000,	// Bitmap 29
0b11100000,
0b11000000,
0b10000000,
0b00000000,
0b00000000,
0b00000000,
0b00000000,
0b00000000,

0b00000000,	// Bitmap 30
0b00000000,
0b00000000,
0b00011111,
0b00111111,
0b01111111,
0b11111111,
0b01111111,
0b00111111,

0b00000000,	// Bitmap 31
0b00000000,
0b00000000,
0b00000000,
0b00000001,
0b00000001,
0b00000001,
0b00000001,
0b00000001,

0b00000001,	//Bitmap 32
0b00000001,
0b00000001,
0b00000001,
0b00000001,
0b00000001,
0b00000001,
0b00000001,
0b00000001,

0b00000001,	// Bitmap 33
0b00000001,
0b00000001,
0b00000001,
0b00000001,
0b00000000,
0b00000000,
0b00000000,
0b00000000
};

//---------------------------------------------------------------------
//   Font fr kleine Ziffern von 0-9 mit Punkt (21*32 Pixel)
//   1.Byte von oben links, dann senkrecht vortlaufen
//   (Nachkommastelle fr groe Temperaturanzeige)
//--------------------------------------------------------------------- 
const uint8_t kleine_ziffer[] PROGMEM =
{
	/* Character Data ".0" */
	0x00,0x00,0x00,0xE0,0x00,0x00,0x00,0xE0,0x00,0x00,0x00,0xE0,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0xFC,0xFF,0xFE,0x7F,0xF8,0x7F,0xFC,0x3F,
	0xF2,0x3F,0xF8,0x9F,0x06,0x00,0x00,0xC0,0x0E,0x00,0x00,0xE0,0x0E,0x00,
	0x00,0xE0,0x0E,0x00,0x00,0xE0,0x0E,0x00,0x00,0xE0,0x0E,0x00,0x00,0xE0,
	0x0E,0x00,0x00,0xE0,0x0E,0x00,0x00,0xE0,0x0E,0x00,0x00,0xE0,0x06,0x00,
	0x00,0xC0,0xF2,0x3F,0xF8,0x9F,0xF8,0x7F,0xFC,0x3F,0xFC,0xFF,0xFE,0x7F,

	/* Character Data ".1" */
	0x00,0x00,0x00,0xE0,0x00,0x00,0x00,0xE0,0x00,0x00,0x00,0xE0,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0xF0,0x3F,0xF8,0x1F,0xF8,0x7F,0xFC,0x3F,0xFC,0xFF,0xFE,0x7F,

	/* Character Data ".2" */
	0x00,0x00,0x00,0xE0,0x00,0x00,0x00,0xE0,0x00,0x00,0x00,0xE0,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFE,0x7F,0x00,0x00,0xFC,0x3F,
	0x02,0x00,0xF9,0x9F,0x06,0x80,0x03,0xC0,0x0E,0x80,0x03,0xE0,0x0E,0x80,
	0x03,0xE0,0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,
	0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,0x06,0x80,
	0x03,0xC0,0xF2,0x3F,0x01,0x80,0xF8,0x7F,0x00,0x00,0xFC,0xFF,0x00,0x00,

	/* Character Data ".3" */
	0x00,0x00,0x00,0xE0,0x00,0x00,0x00,0xE0,0x00,0x00,0x00,0xE0,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x02,0x00,0x01,0x80,0x06,0x80,0x03,0xC0,0x0E,0x80,0x03,0xE0,0x0E,0x80,
	0x03,0xE0,0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,
	0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,0x06,0x80,
	0x03,0xC0,0xF2,0x3F,0xF9,0x9F,0xF8,0x7F,0xFC,0x3F,0xFC,0xFF,0xFE,0x7F,

	/* Character Data ".4" */
	0x00,0x00,0x00,0xE0,0x00,0x00,0x00,0xE0,0x00,0x00,0x00,0xE0,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0xFC,0xFF,0x00,0x00,0xF8,0x7F,0x00,0x00,
	0xF0,0x3F,0x01,0x00,0x00,0x80,0x03,0x00,0x00,0x80,0x03,0x00,0x00,0x80,
	0x03,0x00,0x00,0x80,0x03,0x00,0x00,0x80,0x03,0x00,0x00,0x80,0x03,0x00,
	0x00,0x80,0x03,0x00,0x00,0x80,0x03,0x00,0x00,0x80,0x03,0x00,0x00,0x80,
	0x03,0x00,0xF0,0x3F,0xF9,0x1F,0xF8,0x7F,0xFC,0x3F,0xFC,0xFF,0xFE,0x7F,

	/* Character Data ".5" */
	0x00,0x00,0x00,0xE0,0x00,0x00,0x00,0xE0,0x00,0x00,0x00,0xE0,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0xFC,0xFF,0x00,0x00,0xF8,0x7F,0x00,0x00,
	0xF2,0x3F,0x01,0x80,0x06,0x80,0x03,0xC0,0x0E,0x80,0x03,0xE0,0x0E,0x80,
	0x03,0xE0,0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,
	0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,0x06,0x80,
	0x03,0xC0,0x02,0x00,0xF9,0x9F,0x00,0x00,0xFC,0x3F,0x00,0x00,0xFE,0x7F,

	/* Character Data ".6" */
	0x00,0x00,0x00,0xE0,0x00,0x00,0x00,0xE0,0x00,0x00,0x00,0xE0,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0xFC,0xFF,0xFE,0x7F,0xF8,0x7F,0xFC,0x3F,
	0xF2,0x3F,0xF9,0x9F,0x06,0x80,0x03,0xC0,0x0E,0x80,0x03,0xE0,0x0E,0x80,
	0x03,0xE0,0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,
	0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,0x06,0x80,
	0x03,0xC0,0x02,0x00,0xF9,0x9F,0x00,0x00,0xFC,0x3F,0x00,0x00,0xFE,0x7F,

	/* Character Data ".7" */
	0x00,0x00,0x00,0xE0,0x00,0x00,0x00,0xE0,0x00,0x00,0x00,0xE0,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x02,0x00,0x00,0x00,0x06,0x00,0x00,0x00,0x0E,0x00,0x00,0x00,0x0E,0x00,
	0x00,0x00,0x0E,0x00,0x00,0x00,0x0E,0x00,0x00,0x00,0x0E,0x00,0x00,0x00,
	0x0E,0x00,0x00,0x00,0x0E,0x00,0x00,0x00,0x0E,0x00,0x00,0x00,0x06,0x00,
	0x00,0x00,0xF2,0x3F,0xF8,0x1F,0xF8,0x7F,0xFC,0x3F,0xFC,0xFF,0xFE,0x7F,

	/* Character Data ".8" */
	0x00,0x00,0x00,0xE0,0x00,0x00,0x00,0xE0,0x00,0x00,0x00,0xE0,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0xFC,0xFF,0xFE,0x7F,0xF8,0x7F,0xFC,0x3F,
	0xF2,0x3F,0xF9,0x9F,0x06,0x80,0x03,0xC0,0x0E,0x80,0x03,0xE0,0x0E,0x80,
	0x03,0xE0,0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,
	0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,0x06,0x80,
	0x03,0xC0,0xF2,0x3F,0xF9,0x9F,0xF8,0x7F,0xFC,0x3F,0xFC,0xFF,0xFE,0x7F,

	/* Character Data ".9 */
	0x00,0x00,0x00,0xE0,0x00,0x00,0x00,0xE0,0x00,0x00,0x00,0xE0,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0xFC,0xFF,0x00,0x00,0xF8,0x7F,0x00,0x00,
	0xF2,0x3F,0x01,0x80,0x06,0x80,0x03,0xC0,0x0E,0x80,0x03,0xE0,0x0E,0x80,
	0x03,0xE0,0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,
	0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,0x0E,0x80,0x03,0xE0,0x06,0x80,
	0x03,0xC0,0xF2,0x3F,0xF9,0x9F,0xF8,0x7F,0xFC,0x3F,0xFC,0xFF,0xFE,0x7F,

	/* Character Data ".-" */
	0x00,0x00,0x00,0xE0,0x00,0x00,0x00,0xE0,0x00,0x00,0x00,0xE0,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x01,0x00,0x00,0x80,0x03,0x00,0x00,0x80,0x03,0x00,0x00,0x80,
	0x03,0x00,0x00,0x80,0x03,0x00,0x00,0x80,0x03,0x00,0x00,0x80,0x03,0x00,
	0x00,0x80,0x03,0x00,0x00,0x80,0x03,0x00,0x00,0x80,0x03,0x00,0x00,0x80,
	0x03,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	
	/* Character Data " " */
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00

};

//----------------------------------------------------------
//  Grafik: Thermometer-Symbol (14*32 Pixel)
// 1.Byte oben links, dann senkrecht vortlaufen 
//----------------------------------------------------------
const char thermometer_sym[] PROGMEM =
{
0x0E, // Symbol-Breite
0x04, // Symbol-Hhe in Page (1 Page = 8 Bit)
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0F,0x00,
0x00,0x00,0x80,0x10,0x00,0x00,0x00,0x40,0x26,0x00,0xFF,
0xFF,0x3F,0x4F,0x80,0x00,0x00,0x80,0x5F,0x40,0x00,0xFE,
0xFF,0x5F,0x40,0x00,0xFE,0xFF,0x5F,0x82,0x00,0x00,0x80,
0x5F,0x00,0xFF,0xFF,0x3F,0x4F,0x0C,0x00,0x00,0x40,0x26,
0x12,0x41,0x10,0x84,0x10,0x12,0x41,0x10,0x04,0x0F,0x00,
0x00,0x00,0x00,0x00
};
//----------------------------------------------------------
//  Grafik: Sonnen-Symbol (13*16 Pixel)
// 1.Byte oben links, dann senkrecht vortlaufen 
//----------------------------------------------------------
const char sonne1_sym[] PROGMEM =
{
	// Sonnen-Symbol 1 (volles Symbol)
	0x0D, // Symbol-Breite
	0x01, // Symbol-Hhe in Page (1 Page = 8 Bit)
	0x80,0x00,0x84,0x10,0xC8,0x09,0x30,0x06,0x10,0x04,0x08,0x08,0x0E,
	0x38,0x08,0x08,0x10,0x04,0x30,0x06,0xC8,0x09,0x84,0x10,0x80,0x00
};
const char sonne2_sym[] PROGMEM =
{
	// Sonen-Symbol 2 (gestricheltes Symbol)
    0x0D, // Symbol-Breite
	0x01, // Symbol-Hhe in Page (1 Page = 8 Bit)
	0x00,0x00,0x00,0x00,0x40,0x01,0x10,0x04,0x00,0x00,0x08,0x08,0x00,
	0x00,0x08,0x08,0x00,0x00,0x10,0x04,0x40,0x01,0x00,0x00,0x00,0x00
};
const char sonne3_sym[] PROGMEM =
{
	// Sonen-Symbol 3 (Leeres Symbol)
	0x0D, // Symbol-Breite
	0x01, // Symbol-Hhe in Page (1 Page = 8 Bit)
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};

//----------------------------------------------------------
//  Grafik: Face-Symbol (13*16 Pixel)
// 1.Byte oben links, dann senkrecht vortlaufen 
//----------------------------------------------------------
const char face1_sym[] PROGMEM =
{
	// face 1 (volles Symbol)
	0x0D, // Symbol-Breite
	0x01, // Symbol-Hhe in Page (1 Page = 8 Bit)
	0x00,0x00,0xC4,0x80,0xFE,0xC7,0xFF,0x67,0xFF,0x7F,0xFF,0x7F,0xFF,
	0x3F,0xFF,0x3F,0xFF,0x3F,0xFE,0x23,0xFC,0x01,0x78,0x00,0x00,0x00
};
const char face2_sym[] PROGMEM =
{
	// face 2 (Symbol lschen)
	0x0D, // Symbol-Breite
	0x01, // Symbol-Hhe in Page (1 Page = 8 Bit)
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
};

//----------------------------------------------------------
//  Display-Texte
//----------------------------------------------------------
const char text_0[]  PROGMEM = "                   ";	// leere Zeile (Anzeige lschen)
const char text_1[]  PROGMEM = "-- --:--    --.- C";	// Grundanzeige
const char text_2[]  PROGMEM = "off ";
const char text_3[]  PROGMEM = "aut.";
const char text_4[]  PROGMEM = "man.";
const char text_4a[] PROGMEM = "on  ";

const char text_5[]  PROGMEM = "   Temperaturen   ";	// Menseite "Temperaturen"
const char text_6[]  PROGMEM = " Sollwert: --.- C ";
const char text_7[]  PROGMEM = " Komfort : --.- C ";
const char text_8[]  PROGMEM = " Nacht   : --.- C ";

const char text_8a[] PROGMEM = " Komfort-Funktion ";	// Menseite "Komfort-Funktion"
const char text_8b[] PROGMEM = "Komforttemp: --   ";
const char text_8c[] PROGMEM = "Lnge      : 15 m ";
const char text_8d[] PROGMEM = "Verzogerung: 30 s ";
const char text_8e[] PROGMEM = "Messdauer  : 60 s ";

const char text_9 [] PROGMEM = "  Nachtabsenkung  ";	// Menseite "Nachtabsenkung"
const char text_10[] PROGMEM = "  von: --:-- Uhr  ";
const char text_11[] PROGMEM = "  bis: --:-- Uhr  ";
const char text_11a[]PROGMEM = "  Helligkeit: --  ";

const char text_12[] PROGMEM = "  Datum/Uhrzeit   ";	// Menseite "Datum/Uhrzeit"
const char text_13[] PROGMEM = "Datum  : --.--.-- ";
const char text_14[] PROGMEM = "Uhrzeit: --.--.-- ";
const char text_15[] PROGMEM = "Tag    : --       ";
const char text_15a[]PROGMEM = "Reset  : off      ";


const char text_16[] PROGMEM = "Mo";					// Wochentage
const char text_17[] PROGMEM = "Di";
const char text_18[] PROGMEM = "Mi";
const char text_19[] PROGMEM = "Do";
const char text_20[] PROGMEM = "Fr";
const char text_21[] PROGMEM = "Sa";
const char text_22[] PROGMEM = "So";

const char text_23[] PROGMEM = "Soll";
const char text_24[] PROGMEM = "    ";
const char text_25[] PROGMEM = "ok ";
const char text_26[] PROGMEM = "wait..";

//################### Interrupt Routinen #######################################

//----------------------------------------------------------
//  externer Interrupt vom PIR-Sensor (falling edge)
//----------------------------------------------------------
ISR(INT0_vect)
    {
	pir_flag=1;
    }

//----------------------------------------------------------
//  Timer1-Interupt lalle 20ms
//----------------------------------------------------------
ISR (TIMER1_COMPA_vect)
	{
	 int_counter++;
	 switch(int_counter)
	   {
	     case  1 :
	       cursor_flashing=1;                                  // Cursor-Blinken bei Parametereingaben im Men
	       break;
	     case 25 :
	       cursor_flashing=0;
	       time_flag=2;			                   				// nach 0,5sec Uhrzeit Doppelpunkt ausschalten
	       break;
             case 50 :				                   			// nach 1sec diverse Timer-Funktionen durchfhren
	       sec_count++;
	       if(sec_count >=1){time_flag=1;}                     	// einmal pro Sekunde aktuelle Uhrzeit anzeigen
	       if(sec_count >=7){ds1820_busy_flag=1;sec_count=0;}  	// alle 7sec Temperaturmessung starten
	       if(pir_timer_on ==1){pir_timer++;}                  	// Timer Bewegungssensor
	       if(komfort_timer_on ==1){komfort_timer++;}          	// Timer Komfort-Temperatur
	       if(face_timer_on ==1){face_timer++;}                	// Timer Face-Symbol auf Display
	       if(menue_timer_on==1){menue_timer++;}               	// Menseite nach 5min verlassen
	       if(flash_big_ziffer==1)				   				// Blinkmodus fr groe Ziffern
             {flash_big_ziffer_timer++;ds1820_busy_flag=0;}     // wrend Blinkmodus keine neue Temperaturmessung zulassen
	       if(ds1820_busy_flag==2){ds1820_busy_flag=3;}	   		// ds1820_busy_flag setzen
	       int_counter=0;					   					// Interuppt-Counter von Timer1 auf null		
	       break;
          }
	 }

//----------------------------------------------------------
//  Display-Funktion: init. Display
//----------------------------------------------------------
void initDOGM128()
{
  contrast = 22;
  Xpixel   = 128;
  Ypages   = 8;
  DOGENABLE;
  DOGCOMMAND;
  dogSPIout(DISPSTART + 0);
  xoffset = 0;
  dogSPIout(ADCREVERSE);
  dogSPIout(COMNORMAL);
  dogSPIout(DISPNORMAL);
  dogSPIout(LCDBIAS9);
  dogSPIout(SETPOWERCTRL+7);
  dogSPIout(BOOSTERRATIO);
  dogSPIout(0);
  dogSPIout(REGRESISTOR+7);
  dogSPIout(SETCONTRAST);
  dogSPIout(contrast);
  dogSPIout(STATINDMODE);
  dogSPIout(1);
  dogSPIout(DISPON);
  DOGDISABLE;
}

//----------------------------------------------------------
//  Display-Funktion: Reset Display
//----------------------------------------------------------
void ResetDOG(void)
{
  SetBit(DOGCSPORT,  DOGCS);
  SetBit(DOGSCLPORT, DOGSCL);
  SetBit(DOGSIPORT,  DOGSI);
  SetBit(DOGA0PORT,  DOGA0);
  ClrBit(DOGRESPORT,  DOGRES);
  _delay_us(5);
  SetBit(DOGRESPORT,  DOGRES);
}

//----------------------------------------------------------
//  Display-Funktion: Kontrast einstellen
//----------------------------------------------------------
void DogKontrast(void)
{
  DOGENABLE;
  DOGCOMMAND;
  dogSPIout(SETCONTRAST);
  dogSPIout(contrast);
  DOGDISABLE;
}

//----------------------------------------------------------
//  Display-Funktion: Serielle Bit-Ausgabe aufs Display
//----------------------------------------------------------
void dogSPIout(char out)
	{
  	 char msk;
  	 msk = 0x80;
	 do
   	  { 
	    ClrBit(DOGSCLPORT, DOGSCL);
        if(out & msk)
        SetBit(DOGSIPORT, DOGSI);
        else
        ClrBit(DOGSIPORT, DOGSI);
	    SetBit(DOGSCLPORT, DOGSCL);
        msk >>= 1;
	  }
     while(msk > 0);
	}

//----------------------------------------------------------
//  Display-Funktion: Anzeige lschen 
//----------------------------------------------------------
void clr_disp()
	{
	 inverse=0;
	 for(uint8_t i=0; i<=7; i++)	// Zeilen-Counter
	 {
	 set_disp_pos(i,0);
	 print_text(text_0);			// text_0 ist eine leere Zeile
	 }
	}
//----------------------------------------------------------
//  Display-Funktion: Ausgabeposition bestimmen
//    x_pos 0-127, y_pos 0-7 (Page)
//----------------------------------------------------------
void set_disp_pos(char y_pos, char x_pos)
	{
	col_h=x_pos;
	col_h &=0xF0; 		// low-nibble lschen
	col_h=col_h >>4; 	// Ergebis 4 mal nach rechts shifften und in col_h speichern
	col_l=x_pos;
	col_l &=0x0F; 		// high-nibble lschen und in col_l speichern
	DOGENABLE;			// Display aktivieren
	DOGCOMMAND;			// Display-Kommando senden
	dogSPIout(PAGEADR+y_pos);	// Y-Position (Page-Adr.) bermitteln
	dogSPIout(COLADRH+col_h);	// Col-H Adr (H-Position)
	dogSPIout(COLADRL+col_l);	// Col-L Adr.
	}	

//----------------------------------------------------------
//  Display-Funktion: Einen Character anzeigen
//  (Character in asc_data)
//----------------------------------------------------------
void print_char(uint8_t asc_data)
	{
	DOGENABLE;
	DOGDATA;
	for(uint8_t i=0; i<=6; i++)						// Ein Byte aus Font laden
	 {
	  disp_data=pgm_read_byte(font+(asc_data*6)+i); // ein Byte aus Zeichenfont lesen
	  if(inverse==1)								// wenn inverse=1 dann inverse Darstellung
	  {
	  disp_data = ~disp_data;
	  }
	  dogSPIout(disp_data);
	 }
	DOGDISABLE;
	}

//----------------------------------------------------------
//  Display-Funktion: Text anzeigen
//  Text-Adresse in char
//----------------------------------------------------------
void print_text(const char *addr)
	{
	char zeichen;
	while ((zeichen = pgm_read_byte( addr++ )) != '\0' )  
	    {
	      print_char(zeichen);	// Textzeichen auf Display anzeigen
	    }
	}

//----------------------------------------------------------
//  Display-Funktion: Symbol anzeigen
//  Symbol-Adr. in addr
//  Symbol-Breite in xpos (Anzahl der Pixel)
//  Symbol-Hhe in ypos (Anzahl der Page   1 Page = 8 Bit)
//----------------------------------------------------------
void print_symbol(const char *addr,uint8_t xpos, uint8_t ypos)
	{
	uint8_t pixel_breite=0;						// Pixelbreite Temperatursymbol
	uint8_t pixel_hoehe=0;						// Pixelhhe in Page Temperatursymbol
	pixel_breite=(pgm_read_byte(addr++));
	pixel_hoehe=(pgm_read_byte(addr++));
	uint8_t ypos_start=ypos;						// Startposition der Y-Koordinate
	for(uint8_t i1=0; i1<=pixel_breite-1; i1++)
	 {
	   set_disp_pos(ypos,xpos);						// Display-Position bestimmen
	   for(uint8_t i=0; i<=pixel_hoehe; i++)					// Counter fr Y-Position (Page-Position)
	 	{
	  	  DOGDATA;
		  disp_data=(pgm_read_byte(addr++)); // read Byte from Bitmap-Table
	  	  dogSPIout(disp_data);
	      ypos++;									// ink. Y-Position
	   	  set_disp_pos(ypos,xpos);					// neue Position berechnen		
		}
	   xpos++;
	   ypos=ypos_start;
	  }
	 DOGDISABLE;
	}

//----------------------------------------------------------
//  Display-Funktion: Hauptseite anzeigen
//----------------------------------------------------------
void print_grundanzeige()
	{
	 clr_disp();
	 inverse=1;
	 set_disp_pos(0,0);			// Textposition bestimmen
	 print_text(text_1);
	 set_disp_pos(0,113);
	 print_char(1); 			// Grad-Symbol anzeigen
	 set_disp_pos(2,105);		// Grad-Symbol oberhalb der kleinen Ziffer anzeigen
	 inverse=0;
	 print_char(1); 		
	 print_char('C');
	 set_disp_pos(7,2);			// Temperaturregler-Modus anzeigen
	 inverse=0;
	 print_text(text_2);
	 print_symbol(thermometer_sym,5,2);
	 inverse=1;
	 set_disp_pos(0,85);		// Textposition
	 temp_soll1=temp_soll_eeprom; // Temperatur-Sollwert laden
	 print_temperatur();
	 if(komfort_flag==1 && komfort_timer_on==1) // Komfort-Temperatur aktiv ?
	  {
	   set_disp_pos(0,70);		// wenn ja "K" in Statuszeile anzeigen
	   inverse =1;
	   print_char('K');
	  }			 	 
	 }

//----------------------------------------------------------
//  Display-Funktion: Menseite 1 anzeigen 
//  (Temperatureinstellung)
//----------------------------------------------------------
void print_menue_1()
	{
	 clr_disp();			
	 inverse=1;
	 set_disp_pos(0,0);
	 print_text(text_5);	// Men-Titel anzeigen
	 inverse=0;
	 set_disp_pos(2,0);
	 print_text(text_6);	// Men-Optionen anzeigen
	 set_disp_pos(3,0);
	 print_text(text_7);
	 set_disp_pos(4,0);
	 print_text(text_8);
	 set_disp_pos(2,106);	// Grad-Symbol anzeigen
	 print_char(1);
	 set_disp_pos(3,106);
	 print_char(1);
	 set_disp_pos(4,106);
	 print_char(1);
	 set_disp_pos(2,76);
	 eeprom_busy_wait();		// warte bis EEPROM bereit
	 temp_soll1=eeprom_read_word(&eetemp_soll);	
	 print_temperatur();	// Soll-Temperatur anzeigen
	 set_disp_pos(3,76);
	 temp_soll1=temp_komfort; 	
	 print_temperatur();	// Komfort-Temperatur anzeigen
	 set_disp_pos(4,76);
	 temp_soll1=temp_nacht; 	
	 print_temperatur();	// Nacht-Temperatur anzeigen
	}

//----------------------------------------------------------
//  Display-Funktion: Menseite 2 anzeigen 
//  (Komfort-Funktion)
//----------------------------------------------------------
void print_menue_2()
	{
	 clr_disp();			
	 inverse=1;
	 set_disp_pos(0,0);
	 print_text(text_8a);
	 inverse=0;
	 set_disp_pos(2,0);
	 print_text(text_8b);
	 set_disp_pos(3,0);
	 print_text(text_8c);
	 set_disp_pos(4,0);
	 print_text(text_8d);
	 set_disp_pos(5,0);
	 print_text(text_8e);
	 set_disp_pos(2,91);
	 if(komfort_temp_funktion==1){print_text(text_4a);}
	 else{print_text(text_2);}
	 set_disp_pos(3,7);		// print ""
	 print_char(0x03);
	 set_disp_pos(4,28);	// print ""
	 print_char(0x04);
	}

//----------------------------------------------------------
//  Display-Funktion: Menseite 3 anzeigen
//  (Nachtabsenkung)
//----------------------------------------------------------
void print_menue_3()
	{
	 clr_disp();			
	 inverse=1;
	 set_disp_pos(0,0);
	 print_text(text_9);
	 inverse=0;
	 set_disp_pos(2,0);
	 print_text(text_10);
	 set_disp_pos(2,49);
	 print_digit(nacht_von_std);
	 set_disp_pos(2,70);
	 print_digit(nacht_von_min);
	 set_disp_pos(3,0);
	 print_text(text_11);
	 set_disp_pos(3,49);
	 print_digit(nacht_bis_std);
	 set_disp_pos(3,70);
	 print_digit(nacht_bis_min);
	 set_disp_pos(5,0);
	 print_text(text_11a);
	 set_disp_pos(5,98);
	 print_digit(helligkeitsstufe);
	}
//----------------------------------------------------------
//  Display-Funktion: Menseite 4 anzeigen
//  (Datum/Uhrzeit)
//----------------------------------------------------------
void print_menue_4()
	{
	 clr_disp();			
	 inverse=1;
	 set_disp_pos(0,0);
	 print_text(text_12);
	 inverse=0;
	 set_disp_pos(2,0);
	 print_text(text_13);
	 set_disp_pos(3,0);
	 print_text(text_14);
	 set_disp_pos(4,0);
	 print_text(text_15);
	 set_disp_pos(6,0);
	 print_text(text_15a);
	 set_disp_pos(2,63);
	 print_datum();
	 set_disp_pos(3,63);
	 print_zeit();
	 set_disp_pos(4,63);
	 print_wochentag();
	}

//----------------------------------------------------------
//  Display-Funktion: Kleine Ziffer anzeigen
//  (Nachkommastelle von groer Temperaturanzeige)
//----------------------------------------------------------
void print_kleine_ziffer(char kl_ziffer)
	{
	uint8_t pixel_breite=21;
	uint8_t xpos=100;					// X-Position auf Display
	uint8_t ypos=4;						// Y-Position auf Display
	uint8_t ypos_start=ypos;		// Startposition der Y-Koordinate
	uint16_t table_pos=kl_ziffer * 84;	// Ziffer-Position in der Tabelle
	page=ypos;
	for(uint8_t i1=0; i1<=pixel_breite-1; i1++)
	 {
	   set_disp_pos(ypos,xpos);
	   for(uint8_t i=0; i<=3; i++)			// Counter fr Y-Koordinate
	    {
	      DOGDATA;
	      disp_data=pgm_read_byte(kleine_ziffer+table_pos); // read Byte from Symbole-Table
	      dogSPIout(disp_data);
	      ypos++;
	      table_pos++;
	      set_disp_pos(ypos,xpos);
	    }
	  xpos++;
	  ypos=ypos_start;
	 }
	 DOGDISABLE;
	}

//----------------------------------------------------------
//  Display-Funktion: Groe Ziffer anzeigen
//  (Groer Temperaturanzeige auf Hauptseite)
//----------------------------------------------------------
void print_big_ziffer(char xpos, char data)
	{
	 uint8_t ypos=2;					
	 set_disp_pos(ypos,xpos);	// Position auf Display
	 DOGDATA;
	 uint8_t next_sek=0;
	 for(uint8_t i1=0; i1<=20; i1++)		// 3x Sekmente schreiben dann Page +1
	  {
	    uint8_t seg_nr=pgm_read_byte(sek_tabel+(data*21)+i1); // Sekment-Nr. aus Tabelle laden
	    for(uint8_t i=0; i<=8; i++)	// Ein Byte aus Font laden
	     {
	      disp_data=pgm_read_byte(bitmap_font+(seg_nr*9)+i); // ein Byte aus Zeichenfont lesen
	      dogSPIout(disp_data);
	     }
	    next_sek=next_sek+1;
	    if(next_sek==3)
	     {
	       ypos++;
	       next_sek=0;
		   set_disp_pos(ypos,xpos);
	       DOGDATA;
	     }
	   }
	  DOGDISABLE;		  
	 }	

//----------------------------------------------------------
//  Temperatursensor: Raumtemperatur anzeigen
//  (Funktion wird im Main-Programm aufgerufen)
//----------------------------------------------------------
void temperatur_anzeigen()
     {
	  switch(flash_big_ziffer)
	   {
	     case 1 : flashing_big_ziffer();	// bei Bettigung von "+" oder "-" Taste blinkende groe Ziffer anzeigen
	       break;
	   }
	  
	  if(ds1820_stop_flag==1){return;}		// wenn Stop_Flag=1 keine Messung durchfhren
	  switch(ds1820_busy_flag)				// Status vom Temperatursensor abfragen
	   {
	     case 0 :							// wenn Flag=0 nix machen (keine Messung)
	       break;
             case 1 : ds1820_start();		// wenn Flag=1 Messung starten
	       break;
             case 2 : 						// wenn Flag=2 auf Ende der Messung warten
	       break;
	     case 3 : ds1820_print_temp();      // wenn Flag=3 Temperatur anzeigen
	       break;
       }
     }

//----------------------------------------------------------
//  Temperatursensor: Temperaturmessung starten
//----------------------------------------------------------
void ds1820_start()
     {
       ds1820_reset();               	 // ab hier Temperaturmessung im DS1820 starten
       if(ds1820_err_flag==1)
       {ds1820_busy_flag=0;return;}      // im Fehlerfall Temperaturmessung abbrechen
       ds1820_byte=0xcc;
       ds1820_write_byte();
       ds1820_byte=0x44;
       ds1820_write_byte();
       ds1820_reset();                
       ds1820_byte=0xcc;              
       ds1820_write_byte();
       ds1820_byte=0xbe;
       ds1820_write_byte();
       ds1820_busy_flag=2;		 		// Flag=2 Temperaturmessung luft
     }

//----------------------------------------------------------
//  Temperatursensor: Rest Temperatursensor
//----------------------------------------------------------
void ds1820_reset()
      {
        cli(); 				 			// disabled Interrupt
	    ds1820_err_flag=0;		 		// Error-Flag lschen
	    sens_ddr |= (1<<sensor1);       // Temp1-Leitung auf Ausgang schalten
        sens_port &=~(1<<sensor1);      // Temp1-Leitung auf 0 setzen
        _delay_us(480);                 // 480 usec warten
        sens_ddr &=~(1<<sensor1);       // Temp1-Leitung auf Eingang schalten
        _delay_us(80);                  // nach 80 us (min 60) auf Low-Impuls wom Sensor warten
        if ( sens_inp & (1<<sensor1))   // Sensorstatus prfen 
         {                            
          set_disp_pos(1,0);     	 	// Cursor-Position setzen (2.Zeile 16.Spalte)
          print_char('E');          	// wenn kein Low-Signal vom Sensor dann Error-Meldung ausgeben
          print_char('r');		 		// dann Error-Meldung ausgeben
          print_char('r');		 		// und Messung abbrechen
          ds1820_err_flag=1;		 	// Error-Flag setzen
	      return;			 			// Funktions-Routine beenden
         }
	    sei(); 			         		// enabled Interrupt
        _delay_us(600);                 // 600 us warten (min 480us)
      }

//----------------------------------------------------------
//  Temperatursensor: Ein Bit an den Sensor bertragen
//----------------------------------------------------------
void ds1820_write_bit()
       {
         sens_ddr |= (1<<sensor1);     	// Temp1-Portleitung auf Ausgang konfigurieren
         sens_port &=~(1<<sensor1);     // Temp1-Leitung auf 0 setzen
	    _delay_us(1);			 		// 1us warten
         if (ds1820_bit==1)		 		// wenn Bit=1 dann Portleitung sofort  
          {				 				// auf Eingang schalten	
            sens_ddr &=~(1<<sensor1);
          }
         _delay_us(60);			 		// Wenn Bit=0 60us warten und dann
         sens_ddr &=~(1<<sensor1);      // Portleitung auf Eingang schalten
       }

//----------------------------------------------------------
//  Temperatursensor: Ein Byte an den Sensor bertragen
//----------------------------------------------------------
void ds1820_write_byte()
        {
          uint8_t i;                   	// Bit-Counter (8Bit Integer ohne Vorzeichen)
	      cli(); 			 			// disabled Interrupt
          for (i=0;i<8;i++)             // Zhlerschleife (8 mal) um Byte an DS1820 zu senden
           {
            ds1820_bit=ds1820_byte >>i; // Byte-Inhalt (i=Anzahl)Bits nach rechts schieben und in ds1820_bit speichern
            ds1820_bit &=0x01;          // Bit0 in ds1820_bit maskieren
            ds1820_write_bit();         // Bit in Sensor schreiben
            _delay_us(100);             // 100 usec warten
           }
	      sei(); 			 			// enable Interrupt
        }

//----------------------------------------------------------
//  Temperatursensor: Ein Bit aus Sensor lesen
//----------------------------------------------------------
// Ein Bit vom Temperatursensor1 lesen
void ds1820_read_bit()
        {
          sens_ddr |= (1<<sensor1);    	// Temp1-Port auf Ausgang schalten
          sens_port &=~ (1<<sensor1);  	// Temp1-Leitung auf 0 setzen
	      _delay_us(1);			 		// 1us warten
          sens_ddr &=~ (1<<sensor1);   	// Temp1-Port auf Eingang schalten
          _delay_us(14);                // 14us warten
          if (sens_inp & (1<<sensor1))  // Sensor-Bit prfen
           {
            ds1820_bit=1;               // wenn Sensor-Bit low dann ds1820_bit=0
           }
          else
           {
            ds1820_bit=0;               // wenn Sensor-Bit high dann ds18_bit=1
           }
          _delay_us(45);		 		// 45us warten
        }
//----------------------------------------------------------
//  Temperatursensor: Ein Byte aus Sensor lesen
//----------------------------------------------------------
void ds1820_read_byte()
         {
           uint8_t i=8, n=0;            // init. Bit-Counter
	       cli(); 						// disabled Interuppt
           while(i--)
	        {
	          ds1820_read_bit();		// Read Sensor-Bit
	          n>>=1;					// Shift one position right and store value
	          n|=(ds1820_bit<<7);
	        }
	      ds1820_byte=n;				// Byte speichern
	      sei(); 						// enable Interrupt
         }

//----------------------------------------------------------
//  Temperatursensor: Temperaturwert aus Sensor lesen
//  und als 16Bit-Wert speichern (0,1 Grad Auflsung)
//----------------------------------------------------------
void ds1820_read_temp()
		{
         uint8_t i;               // Byte-Counter (8Bit Integer ohne Vorzeichen)
         for (i=1;i<9;i++)        // Zhlerschleife (8 mal) um Register-Bytes aus DS1820 zu lesen
		 {
		  ds1820_read_byte();
		  if(i==1){ds1820_lsb=ds1820_byte;}
		  if(i==2){ds1820_msb=ds1820_byte;}
		  if(i==7){ds1820_countremain=ds1820_byte;}
		  if(i==8){ds1820_countperc=ds1820_byte;}
		 } // Ab hier 0,1 Grad Temperaturauflsung berechnen (im ds1820_lsb wird Bit0 (0.5 Grad-Bit) gelscht)		 	 
         uint16_t temp_read = (ds1820_msb * 256) + ds1820_lsb;
         ds1820_temperatur=(temp_read & 0xFFFE)*50 - 25 + ((ds1820_countperc-ds1820_countremain)*100)/ds1820_countperc;
         temperatur=ds1820_temperatur;
		 if(ds1820_temperatur>0x7FFF){ds1820_temperatur=~ds1820_temperatur;ds1820_minus=1;}
	     else
		  {
		   ds1820_minus=0;
		  }
		 }

//----------------------------------------------------------
//  Temperatursensor: Temperaturwert auf Hauptseite anzeigen
//----------------------------------------------------------
void ds1820_print_temp()
     {
       ds1820_read_temp();				// Temperaturwert vom Sensor auslesen
       int16_t zahl=ds1820_temperatur; 	// temporrer Zwischenspeicher fr Berechnungen (16Bit Integer ohne Vorzeichen)
	   int8_t digit=0;
	   digit=zahl/10000;				// 10000er Stelle verwerfen
	   zahl=zahl % 10000;
	   digit=zahl/1000;
	   if(digit >=1)					// wenn Zahl >0 dann Zahl anzeigen
	    {
	      print_big_ziffer(38,digit);	// 1000er Stelle anzeigen (Temperaturwert Zehner)
	    }
	   else
        {
          print_big_ziffer(38,11);		// wenn Zahl <1 dann Leerzeichen anzeigen
        }
	   zahl=zahl % 1000;
	   digit=zahl/100;
	   print_big_ziffer(70,digit);		// 100er Stelle anzeigen (Temperaturwert Einer)
	   zahl=zahl % 100;
	   digit=zahl/10;
	   print_kleine_ziffer(digit);		// 10er Stelle anzeigen (Temperaturwert Nachkommastelle)
	   ds1820_busy_flag=0;			    // Flag fr neue Messung lschen (wird in Interuppt-Routine gesetzt)
	 }

//----------------------------------------------------------
//  Relais : Relais in Abhngigkeit der gemessenen Temperatur
//           ein oder ausschalten
//----------------------------------------------------------
void relais_test()
	{
	if(regler==1)							// wenn Temperaturregler automatisch
	{
	 if(temperatur>=temp_soll1+25)			// Hysterese +2,5 Grad
	   {relais_off;heizen_flag=0;print_symbol(sonne3_sym,23,4);;} // Sonnen-Symbol lschen

	 if(temperatur<=temp_soll1-25)		    // Hysterese -2,5 Grad
	   {relais_on;heizen_flag=1;}
	}
	}

//----------------------------------------------------------
//  I2C - Funktion : I2C initialisieren
//----------------------------------------------------------
void i2c_init()
	{
	 _delay_ms(100);	// zur Sicherheit etwas warten
	 scl_out;			// SCL-Portleitung auf Ausgang
	 sda_out;			// SDA-Portleitung auf Ausgang
	 scl_high;			// SDA-Pin auf low schalten
	 sda_high;			// SCL-Pin auf low schalten
	 _delay_ms(10);
	}

//----------------------------------------------------------
//  I2C - Funktion : I2C bertragung starten
//----------------------------------------------------------
void i2c_start()
	{
	scl_high;
	sda_out;
	sda_high;
	_delay_us(1);
	sda_low;
	}

//----------------------------------------------------------
//  I2C - Funktion : I2C bertragung stoppen
//----------------------------------------------------------
void i2c_stop()
	{
	 sda_out;		// SCL-Portleitung auf Ausgang
	 scl_high;		// SCL-Pin auf high schalten
	 _delay_us(1);
	 sda_high;		// SDA-Pin auf high schalten
	}

//----------------------------------------------------------
//  I2C - Funktion : Ein Byte senden
//----------------------------------------------------------
void i2c_write(unsigned char data)
	{
	  scl_low;
	  sda_out;						// SDA-Leitung auf Ausgang
	  _delay_us(10);				// 5us warten (CLK-Frequenz ca.100KHz)
	  unsigned char mask=0x80, i;	// init. Bit-Maske und Counter
	  for(i=0;i<8;i++)
	   {
	     if(data & mask)	
	      {
		    sda_high;			// Bit=1 SDA-Leitung auf high
	      }
	     else
	      {
		    sda_low;			// Bit=0 SDA-Leitung auf low
	      }
	     scl_high;				// SCL-Leitung auf high
	     _delay_us(10); 			// 5us warten (CLK-Frequenz ca.100KHz)
	     scl_low;				// SCL-Leitung auf low
		 _delay_us(10);			// 5us warten
		 mask >>=1;				// nchstes Bit
	   }
	   sda_in;					// SDA-Leitung auf Eingang fr Acknowledge vom Slave
	   _delay_us(10);			// etwas warten
	   if(PINE & (1 << PINE5))	// Acknowledge-Signal lesen
		{
//		 led2_on();				// wenn high dann Fehler Led2 on
		}
	   scl_high;				// 9. SCL-Clocksignal fr Acknowledge
	   _delay_us(10);
	   scl_low;
	  }

//----------------------------------------------------------
//  I2C - Funktion : Ein Byte empfangen
//----------------------------------------------------------
void i2c_read()
	{
	 uint8_t wert1=0, wert3;	// Counter und Mask
	 scl_low;					// SCL-Leitung low
	 sda_in;					// SDA-Leitung auf Eingang schalten
	 _delay_us(10);				// etwas warten 
	 for(wert3=1;wert3<9; wert3++)	// Bit-Counter +1
	 {
	  wert1=wert1<<1;				// Datenbit links shiften
	  if(PINE & (1 << PINE5))		// Datenbit lesen
		{
		 wert1=wert1+1;
		}
		scl_high;				// SCL-Leitung einmal takten
		_delay_us(10);			
		scl_low;
		_delay_us(10);
     }
	 sda_out;					// no_Acknowldge an Slave senden
	 sda_low;
	 scl_high;
	 _delay_us(10);
	 scl_low;
	testbyte=wert1;				// Datenbyte speichern
	}

//----------------------------------------------------------
//  RealTimeClock : init RTC
//----------------------------------------------------------
void init_rtc()
	{
	_delay_ms(200);				// etwas warten bis RTC-Takt stabiel
	i2c_start();				// VL-Flag (voltage-low detector) prfen
	i2c_write(0xA2);
	i2c_write(0x02);			// Sekunden-Reg. lesen (Bit7 = VL-Flag)
	i2c_start();
	i2c_write(0xA3);
	i2c_read();					// read Byte
	i2c_stop();
	_delay_ms(20);
	if(testbyte >=0x80)			// prfe Batteriestatus (FL-Flag=0 Batterie ok)
	{							// wenn VL-Flag=1 dann Uhrzeit auf 00:00:00
	jahr=10;					// Datum und Uhrzeit setzen
	monat=1;
	tag=1;
	wochentag=0;
	stunde=0;
	minute=0;
	sekunde=0;
	jahr=((((jahr)/10)<<4)|((jahr)%10));	// Dezimal in BCD umwandeln
	monat=((((monat)/10)<<4)|((monat)%10));
	tag=((((tag)/10)<<4)|((tag)%10));
	stunde=((((stunde)/10)<<4)|((stunde)%10));
	minute=((((minute)/10)<<4)|((minute)%10));
	sekunde=((((sekunde)/10)<<4)|((sekunde)%10));
	i2c_start();
	i2c_write(0xA2);
	i2c_write(0x00);
	i2c_write(0);			// Reg. 0x00 	control_status_1
	i2c_write(0);			// Reg. 0x01 	control_status_2
	i2c_write(sekunde);		// Reg. 0x02 	VL_seconds
	i2c_write(minute);		// Reg. 0x03	minutes
	i2c_write(stunde);		// Reg. 0x04	hours
	i2c_write(tag);			// Reg. 0x05	days
	i2c_write(wochentag);	// Reg. 0x06	weekdays
	i2c_write(monat);		// Reg. 0x07	century_months
	i2c_write(jahr);		// Reg. 0x08	years
	i2c_write(0);
	i2c_write(0);
	i2c_write(0);
	i2c_write(0);
	i2c_write(0);
	i2c_write(0);
	i2c_write(0);
	i2c_stop();
	_delay_ms(20);
	i2c_start();			// CLK_OUT auf 1Hz fr Sekunden-Interrupt im ATmega
	i2c_write(0xA2);
	i2c_write(0x0D);
	i2c_write(0b10000011);
	i2c_stop();
	_delay_ms(20);
	}
	}

//----------------------------------------------------------
//  RealTimeClock : Genderte Uhrzeit an RTC bertragen
//----------------------------------------------------------
void write_time()
	{
	stunde=((((digit_stunde)/10)<<4)|((digit_stunde)%10));
	minute=((((digit_minute)/10)<<4)|((digit_minute)%10));
	sekunde=((((digit_sekunde)/10)<<4)|((digit_sekunde)%10));
	i2c_start();
	i2c_write(0xA2);
	i2c_write(0x00);
	i2c_write(0);			// Reg. 0x00 	control_status_1
	i2c_write(0);			// Reg. 0x01 	control_status_2
	i2c_write(sekunde);		// Reg. 0x02 	VL_seconds
	i2c_write(minute);		// Reg. 0x03	minutes
	i2c_write(stunde);		// Reg. 0x04	hours
	i2c_stop();
	}

//----------------------------------------------------------
//  RealTimeClock : Gendertes Datum und Wochentag an RTC
//                  bertragen
//----------------------------------------------------------
void write_rtc_date()
	{
	tag=((((tag)/10)<<4)|((tag)%10));
	monat=((((monat)/10)<<4)|((monat)%10));
	jahr=((((jahr)/10)<<4)|((jahr)%10));
	i2c_start();
	i2c_write(0xA2);
	i2c_write(0x05);
	i2c_write(tag);
	i2c_write(wochentag);
	i2c_write(monat);
	i2c_write(jahr);
	i2c_stop();
	}

//----------------------------------------------------------
//  RealTimeClock : Aktuelle Uhrzeit und Datum aus RTC lesen
//----------------------------------------------------------
void read_time()
	{
	if(time_flag==1)
	{
	time_flag=0;
	i2c_start();
	i2c_write(0xA2);
	i2c_write(0x02);
	i2c_start();
	i2c_write(0xA3);
	i2c_read();
	_delay_us(20);
	testbyte &=0b01111111; 	// Bit7 lschen			
	sekunde=( testbyte%16 + 10 * (testbyte/16)); // BCD in Hex
	i2c_read();
	_delay_us(20);
	testbyte &=0b01111111;	// Bit7 lschen	
	minute=( testbyte%16 + 10 * (testbyte/16));
	i2c_read();
	_delay_us(20);
	testbyte &=0b00111111;	// Bit7+6 lschen				
	stunde=( testbyte%16 + 10 * (testbyte/16));
	i2c_read();				// Datums-Tag lesen
	_delay_us(20);
	testbyte &=0b00111111;	// Bit7+6 lschen
	tag=( testbyte%16 + 10 * (testbyte/16));
	i2c_read();				// Wochen-Tag lesen
	_delay_us(20);
	testbyte &=0b00000111;	// Bit7-3 lschen
	wochentag=testbyte;
	i2c_read();				// Datums-Monat lesen
	_delay_us(20);
	testbyte &=0b00011111;	// Bit7-5 lschen
	monat=( testbyte%16 + 10 * (testbyte/16));
	i2c_read();				// Datums-Jahr lesen
	_delay_us(20);
	jahr=( testbyte%16 + 10 * (testbyte/16));
	i2c_stop();
	inverse=1;
	set_disp_pos(0,0);
	print_wochentag();
	print_realtime();
	if (heizen_flag==1)	// Wenn Relais ein Sonnen-Symbol anzeigen
	  {
	   print_symbol(sonne1_sym,23,4);
	  }
	}
	if(time_flag==2)
	 {
	  time_flag=0;
	  inverse=1;
	  set_disp_pos(0,34);
	  print_char(' ');		// Doppelpunkt nach 1/2 Sekunde lschen
	  if (heizen_flag==1)	// Wenn Relais ein Sonnen-Symbol anzeigen
	  {
	   print_symbol(sonne2_sym,23,4);
	  }
	 } 
	}

//----------------------------------------------------------
//  RealTimeClock : Aktuelle Uhrzeit anzeigen
//----------------------------------------------------------
void print_realtime()
	{
     uint8_t digit=0;		// temporrer Zwischenspeicher fr Berechnungen 
	 inverse=1;				// Text inverse anzeigen
	 set_disp_pos(0,21);
	 digit=stunde/10;
	 if(digit >0){print_char(digit+48);}
	 else{print_char(' ');}
	 digit=stunde % 10;
	 print_char(digit+48);
	 print_char(':');
	 digit=minute/10;
	 print_char(digit+48);
	 digit=minute % 10;
	 print_char(digit+48);
	}

//----------------------------------------------------------
//  RealTimeClock : Aktuelle Uhrzeit holen
//----------------------------------------------------------
void get_realtime()
	{
	i2c_start();
	i2c_write(0xA2);
	i2c_write(0x02);
	i2c_start();
	i2c_write(0xA3);
	i2c_read();
	_delay_us(20);
	testbyte &=0b01111111; 	// Bit7 lschen			
	sekunde=( testbyte%16 + 10 * (testbyte/16)); // BCD in Hex
	i2c_read();
	_delay_us(20);
	testbyte &=0b01111111;	// Bit7 lschen	
	minute=( testbyte%16 + 10 * (testbyte/16));
	i2c_read();
	_delay_us(20);
	testbyte &=0b00111111;	// Bit7+6 lschen				
	stunde=( testbyte%16 + 10 * (testbyte/16));
	i2c_read();				// Datums-Tag verwerfen
	_delay_us(20);
	i2c_read();				// Wochen-Tag lesen
	_delay_us(20);
	testbyte &=0b00000111;	// Bit7-3 lschen
	wochentag=testbyte;	
	i2c_stop();
	}

//----------------------------------------------------------
//  Aktuelle Datum holen
//----------------------------------------------------------
void get_realdate()
	{
	 i2c_start();
	 i2c_write(0xA2);
	 i2c_write(0x05);
	 i2c_start();
	 i2c_write(0xA3);
	 i2c_read();				// Datums-Tag lesen
	 _delay_us(20);
	 testbyte &=0b00111111;	// Bit7+6 lschen
	 tag=( testbyte%16 + 10 * (testbyte/16));
	 i2c_read();				// Wochen-Tag verwerfen
	 _delay_us(20);
	 i2c_read();				// Datums-Monat lesen
	 _delay_us(20);
	 testbyte &=0b00011111;	// Bit7-5 lschen
	 monat=( testbyte%16 + 10 * (testbyte/16));
	 i2c_read();				// Datums-Jahr lesen
	 _delay_us(20);
	 jahr=( testbyte%16 + 10 * (testbyte/16));
	 i2c_stop();
	}

//----------------------------------------------------------
//  Uhrzeit anzeigen
//----------------------------------------------------------
void print_zeit()
	{
	 get_realtime();
	 print_digit(stunde);
	 print_char(':');
	 print_digit(minute);
	 print_char(':');
	 print_digit(sekunde);
	}
//----------------------------------------------------------
//  Datum anzeigen
//----------------------------------------------------------
void print_datum()
	{
	 get_realdate();
	 print_digit(tag);
	 print_char('.');
	 print_digit(monat);
	 print_char('.');
	 print_digit(jahr);
	}

//----------------------------------------------------------
//  Zweistellige Ziffer anzeigen (Wert in digit)
//----------------------------------------------------------
void print_digit(uint8_t digit)
	{
	 uint8_t digit_x;			// temporrer Zwischenspeicher fr Berechnung
	 digit_x=digit/10;
	 if(digit_x >0){print_char(digit_x+48);}
	 else{print_char('0');}
	 digit_x=digit % 10;
	 print_char(digit_x+48);
	}

//----------------------------------------------------------
//  Wochentag anzeigen
//----------------------------------------------------------
void print_wochentag()
	{
 	 switch(wochentag)
     {
       case 0 :  print_text(text_16);	// Montag
              break;
       case 1 :  print_text(text_17);	// Dienstag
              break;
       case 2 :  print_text(text_18);	// Mittwoch
              break;
       case 3 :  print_text(text_19);	// Donnerstag
              break;
       case 4 :  print_text(text_20);	// Freitag
              break;
       case 5 :  print_text(text_21);	// Samstag
              break;
       case 6 :  print_text(text_22);	// Sonntag
              break;
	 }
	}

//----------------------------------------------------------
//  Temperaturregler auto/off schalten
//----------------------------------------------------------
void regler_switch()
	{
	  switch(regler)
	   {
	     case 0 : set_regler_auto();
		   break;
         case 1 : set_regler_off();
		   break;
       }
    }	

//----------------------------------------------------------
//  Men-Funktion: Menseiten durchblttern
//----------------------------------------------------------
void inc_menue_site()
	{
	  menue_timer_on=0;cursor_on=0;cursor_flash();								// Men-Timer stop, Cursor lschen
	  menue_site++;
	  switch(menue_site)														// Taste "<<" (Men links blttern)
	  {
		case 1 : print_menue_1(); menue_site=1;menue_timer_on=1;menue_timer=0;				// Men "Temperaturen" anzeigen
		  break;
		case 2 : print_menue_2(); menue_site=2;menue_timer_on=1;menue_timer=0;				// Men "Komfort-Funktion" anzeigen
		  break;
		case 3 : print_menue_3(); menue_site=3;menue_timer_on=1;menue_timer=0;				// Men "Nachtabschaltung" anzeigen
		  break;
		case 4 : print_menue_4(); menue_site=4;menue_timer_on=1;menue_timer=0;				// Men "Datum/Uhrzeit" anzeigen
		  break;
		case 5 : print_grundanzeige(); sensor_flag=1;menue_site=0; // Grundanzeige anzeigen
				 ds1820_busy_flag=1;ds1820_stop_flag=0;
		         if(regler==0){set_regler_off();}else{set_regler_auto();} 				// Regler-Status anzeigen
		  break;
	  }
	}

//----------------------------------------------------------
//  Men-Funktion: Cursor-Blinken einschalten
//----------------------------------------------------------
void set_cursor_flashing()
	{
	  cursor_on=1;				// Cursor-Blinken ein
	  cursor_timer=0;
	  menue_timer=0;			// Men-Timer-Wert auf 0
	  menue_timer_on=1;			// Men-Timer starten
	}

//----------------------------------------------------------
//  inc/dec Soll-Temperatur ber "+" Taste in Grundanzeige
//----------------------------------------------------------
void inc_temp_sollwert()
	{
	 if (temp_soll <=2990)
	 {
	  temp_soll=temp_soll+10;
	  temp_soll1=temp_soll;
	  inverse=1;set_disp_pos(0,85);
	  print_temperatur();			// genderte Soll-Temperatur in Statuszeile anzeigen
	  print_big_temperatur();
	  cursor_timer=1;
	  cursor_flashing=1;
	  set_disp_pos(0,77);
	  inverse=1;
	  print_char(' ');
	  set_disp_pos(3,99);
	  inverse=0;
	  print_text(text_23);			// Text "Soll" anzeigen
	 }
	}

void dec_temp_sollwert()
	{
	if (temp_soll >=1010)
	 {
	  temp_soll=temp_soll-10;
	  temp_soll1=temp_soll;
	  inverse=1;set_disp_pos(0,85);
	  print_temperatur();			// genderte Soll-Temperatur in Statuszeile anzeigen
	  print_big_temperatur();
	  cursor_timer=1;
	  cursor_flashing=1;
	  set_disp_pos(0,77);
	  inverse=1;
	  print_char(' ');
	  set_disp_pos(3,99);
	  inverse=0;
	  print_text(text_23);			// Text "Soll" anzeigen
	 }
	}

//----------------------------------------------------------
//  Temperaturwert in groen Ziffern anzeigen
//----------------------------------------------------------
void print_big_temperatur()
	{
	 uint16_t digit;
	 uint16_t temp_soll1a;
	 temp_soll1a=temp_soll1;
	 digit=temp_soll1a/1000;
	 if(digit >0){print_big_ziffer(38,digit);}
	 else{print_big_ziffer(38,11);}
	 temp_soll1a=temp_soll1a%1000;
	 digit=temp_soll1a/100;
	 print_big_ziffer(70,digit);
	 temp_soll1a=temp_soll1a%100;
	 digit=temp_soll1a/10;
	 print_kleine_ziffer(digit);
	}
//----------------------------------------------------------
//  blinkende groe Ziffern
//----------------------------------------------------------
void flashing_big_ziffer()
	{

	 if(flash_big_ziffer_timer <4)
	  {
	    ds1820_stop_flag=1;
		set_disp_pos(3,99);
	    inverse=0;
	    print_text(text_23);		// Text "Soll" anzeigen
		if(cursor_flashing==0)
		{
		print_big_ziffer(38,11);	// Ziffern lschen
		print_big_ziffer(70,11);
		print_kleine_ziffer(11);
		}
		if(cursor_flashing==1)
		{
	      print_big_temperatur();
		}
	  }
	  else
	  {
	   flash_big_ziffer=0;			// Blinken ausschalten
	   print_big_temperatur();		// letzten Temperaturwert anzeigen
	   ds1820_stop_flag=0;
	   ds1820_busy_flag=1;		    // aktuellen Temperaturwert anzeigen
	   set_disp_pos(3,99);			// Text "Soll" lschen
	   inverse=0;
	   print_text(text_24);
	  }
	}

//----------------------------------------------------------
//  Men-Funktion: blinkender Cursor positionieren
//----------------------------------------------------------
void menue_temperaturen_sollwert()
	{
      set_disp_pos(4,69);		// Display-Position
	  inverse=0;
	  print_char(' ');			// Cursor-Symbol auf alter Position lschen
	  cursor_ypos=2;			// neue Cursor-Position setzen
	  cursor_xpos=69;
	  set_cursor_flashing();	// Cursor-Blinken einschalten	  
	}
void menue_temperaturen_komfort()
	{
      set_disp_pos(2,69);		// Display-Position
	  inverse=0;
	  print_char(' ');			// Cursor-Symbol auf alter Position lschen
	  cursor_ypos=3;			// neue Cursor-Position setzen
	  cursor_xpos=69;
	  set_cursor_flashing();	  
	}
void menue_temperaturen_nacht()
	{
      set_disp_pos(3,69);		// Display-Position
	  inverse=0;
	  print_char(' ');			// Cursor-Symbol auf alter Position lschen
	  cursor_ypos=4;			// neue Cursor-Position setzen
	  cursor_xpos=69;
	  set_cursor_flashing();	  
	}

void menue_komfort_temp()
	{
      set_disp_pos(5,84);		// Display-Position
	  inverse=0;
	  print_char(' ');			// Cursor-Symbol auf alter Position lschen
	  cursor_ypos=2;			// neue Cursor-Position setzen
	  cursor_xpos=84;
	  set_cursor_flashing();	  
	}
void menue_komfort_laenge()
	{
      set_disp_pos(2,84);		// Display-Position
	  inverse=0;
	  print_char(' ');			// Cursor-Symbol auf alter Position lschen
	  cursor_ypos=3;			// neue Cursor-Position setzen
	  cursor_xpos=84;
	  set_cursor_flashing();	  
	}
void menue_komfort_verzoegerung()
	{
      set_disp_pos(3,84);		// Display-Position
	  inverse=0;
	  print_char(' ');			// Cursor-Symbol auf alter Position lschen
	  cursor_ypos=4;			// neue Cursor-Position setzen
	  cursor_xpos=84;
	  set_cursor_flashing();	  
	}
void menue_komfort_messdauer()
	{
      set_disp_pos(4,84);		// Display-Position
	  inverse=0;
	  print_char(' ');			// Cursor-Symbol auf alter Position lschen
	  cursor_ypos=5;			// neue Cursor-Position setzen
	  cursor_xpos=84;
	  set_cursor_flashing();	  
	}

void menue_nachtabsenkung_von_std()
	{
      set_disp_pos(5,91);		// Display-Position
	  inverse=0;
	  print_char(' ');			// Cursor-Symbol auf alter Position lschen
	  cursor_ypos=2;			// neue Cursor-Position setzen
	  cursor_xpos=42;
	  set_cursor_flashing();	  
	}

void menue_nachtabsenkung_von_min()
	{
      set_disp_pos(2,42);		// Display-Position
	  inverse=0;
	  print_char(' ');			// Cursor-Symbol auf alter Position lschen
	  cursor_ypos=2;			// neue Cursor-Position setzen
	  cursor_xpos=63;
	  set_cursor_flashing();	  
	}

void menue_nachtabsenkung_bis_std()
	{
      set_disp_pos(2,63);		// Display-Position
	  inverse=0;
	  print_char(':');			// Cursor-Symbol auf alter Position lschen
	  cursor_ypos=3;			// neue Cursor-Position setzen
	  cursor_xpos=42;
	  set_cursor_flashing();	  
	}

void menue_nachtabsenkung_bis_min()
	{
      set_disp_pos(3,42);		// Display-Position
	  inverse=0;
	  print_char(' ');			// Cursor-Symbol auf alter Position lschen
	  cursor_ypos=3;			// neue Cursor-Position setzen
	  cursor_xpos=63;
	  set_cursor_flashing();	  
	}

void menue_nachtabsenkung_licht()
	{
      set_disp_pos(3,63);		// Display-Position
	  inverse=0;
	  print_char(':');			// Cursor-Symbol auf alter Position lschen
	  cursor_ypos=5;			// neue Cursor-Position setzen
	  cursor_xpos=91;
	  set_cursor_flashing();	  
	}

void menue_datumuhrzeit_datum_tag()
	{
      set_disp_pos(6,56);		// Display-Position
	  inverse=0;
	  print_char(' ');			// Cursor-Symbol auf alter Position lschen
	  cursor_ypos=2;			// neue Cursor-Position setzen
	  cursor_xpos=56;
	  set_cursor_flashing();
	}

void menue_datumuhrzeit_datum_monat()
	{
      set_disp_pos(2,56);		// Display-Position
	  inverse=0;
	  print_char(' ');			// Cursor-Symbol auf alter Position lschen
	  cursor_ypos=2;			// neue Cursor-Position setzen
	  cursor_xpos=77;
	  set_cursor_flashing();
	}

void menue_datumuhrzeit_datum_jahr()
	{
      set_disp_pos(2,77);		// Display-Position
	  inverse=0;
	  print_char(':');			// Cursor-Symbol auf alter Position lschen
	  cursor_ypos=2;			// neue Cursor-Position setzen
	  cursor_xpos=98;
	  set_cursor_flashing();
	}

void menue_datumuhrzeit_uhrzeit_std()
	{
      set_disp_pos(2,98);		// Display-Position
	  inverse=0;
	  print_char(':');			// Cursor-Symbol auf alter Position lschen
	  cursor_ypos=3;			// neue Cursor-Position setzen
	  cursor_xpos=56;
	  set_cursor_flashing();
	}

void menue_datumuhrzeit_uhrzeit_min()
	{
      set_disp_pos(3,56);		// Display-Position
	  inverse=0;
	  print_char(' ');			// Cursor-Symbol auf alter Position lschen
	  cursor_ypos=3;			// neue Cursor-Position setzen
	  cursor_xpos=77;
	  set_cursor_flashing();
	}

void menue_datumuhrzeit_uhrzeit_sek()
	{
      set_disp_pos(3,77);		// Display-Position
	  inverse=0;
	  print_char(':');			// Cursor-Symbol auf alter Position lschen
	  cursor_ypos=3;			// neue Cursor-Position setzen
	  cursor_xpos=98;
	  set_cursor_flashing();
	}

void menue_datumuhrzeit_tag()
	{
      set_disp_pos(3,98);		// Display-Position
	  inverse=0;
	  print_char(':');			// Cursor-Symbol auf alter Position lschen
	  cursor_ypos=4;			// neue Cursor-Position setzen
	  cursor_xpos=56;
	  set_cursor_flashing();	  
	}

void menue_datumuhrzeit_reset()
	{
      set_disp_pos(4,56);		// Display-Position
	  inverse=0;
	  print_char(' ');			// Cursor-Symbol auf alter Position lschen
	  cursor_ypos=6;			// neue Cursor-Position setzen
	  cursor_xpos=56;
	  set_cursor_flashing();	  
	}

//----------------------------------------------------------
//  Men-Funktion: Men-Werte ndern
//----------------------------------------------------------
void inc_temperaturen_sollwert()		// Temperatur-Sollwert + 0,1 C
	{
	if (temp_soll <=2990)
	 {
	  temp_soll_eeprom=temp_soll_eeprom+10;
	  temp_soll1=temp_soll_eeprom;
	  inverse=0;set_disp_pos(2,76);
	  print_temperatur();
	  change_flag=1;
	 }
	}
void dec_temperaturen_sollwert()		// Temperatur-Sollwert - 0,1 C
	{
	if (temp_soll >=1010)
	 {
	  temp_soll_eeprom=temp_soll_eeprom-10;
	  temp_soll1=temp_soll_eeprom;
	  inverse=0;set_disp_pos(2,76);
	  print_temperatur();
	  change_flag=1;
	 }
	}

void inc_temperaturen_komfort()
	{
	 if (temp_komfort <=2990)
	 {
	  temp_komfort=temp_komfort+10;
	  temp_soll1=temp_komfort;
	  inverse=0;set_disp_pos(3,76);
	  print_temperatur();
	  change_flag=1;
	 }
	}
void dec_temperaturen_komfort()
	{
	 if (temp_komfort >=1010)
	 {
	  temp_komfort=temp_komfort-10;
	  temp_soll1=temp_komfort;
	  inverse=0;set_disp_pos(3,76);
	  print_temperatur();
	  change_flag=1;
     }
	}

void inc_temperaturen_nacht()
	{
	 if (temp_nacht <=2990)
	 {
	  temp_nacht=temp_nacht+10;
	  temp_soll1=temp_nacht;
	  inverse=0;set_disp_pos(4,76);
	  print_temperatur();
	  change_flag=1;
	 }
	}
void dec_temperaturen_nacht()
	{
	 if (temp_nacht >=1010)
	 {
	  temp_nacht=temp_nacht-10;
	  temp_soll1=temp_nacht;
	  inverse=0;set_disp_pos(4,76);
	  print_temperatur();
	  change_flag=1;
     }
	}

void komfortfunktion_komforttemp_on()
	{
     komfort_temp_funktion=1;			// Komfort-Funktion ein
	 komfort_flag=1;
	 print_text(text_4a);			// Textanzeige: "on"
	 change_flag=1;
	}
void komfortfunktion_komforttemp_off()
	{
	 komfort_temp_funktion=0;			// Komfort-Funktion aus
	 komfort_flag=0;
	 komfort_timer_on=0;
	 komfort_timer=0;
	 print_text(text_2);			// Textanzeige "off"
	 change_flag=1;
	}

void inc_nachtabsenkung_von_std()
	{
	  digit_stunde++;
	  if(digit_stunde >23){digit_stunde=0;}
	  print_digit(digit_stunde);
	  nacht_von_std=digit_stunde;
	  change_flag=1;
	}
void dec_nachtabsenkung_von_std()
	{
	  if(digit_stunde ==0){digit_stunde=24;}
	  digit_stunde--;
	  print_digit(digit_stunde);
	  nacht_von_std=digit_stunde;
	  change_flag=1;
	}

void inc_nachtabsenkung_von_min()
	{
	  digit_minute++;
	  if(digit_minute >59){digit_minute=0;}
	  print_digit(digit_minute);
	  nacht_von_min=digit_minute;
	  change_flag=1;
	}
void dec_nachtabsenkung_von_min()
	{
	  if(digit_minute ==0){digit_minute=60;}
	  digit_minute--;
	  print_digit(digit_minute);
	  nacht_von_min=digit_minute;
	  change_flag=1;
	}
void inc_nachtabsenkung_bis_std()
	{
	  digit_stunde++;
	  if(digit_stunde >23){digit_stunde=0;}
	  print_digit(digit_stunde);
	  nacht_bis_std=digit_stunde;
	  change_flag=1;
	}
void dec_nachtabsenkung_bis_std()
	{
	  if(digit_stunde ==0){digit_stunde=24;}
	  digit_stunde--;
	  print_digit(digit_stunde);
	  nacht_bis_std=digit_stunde;
	  change_flag=1;
	}

void inc_nachtabsenkung_bis_min()
	{
	  digit_minute++;
	  if(digit_minute >59){digit_minute=0;}
	  print_digit(digit_minute);
	  nacht_bis_min=digit_minute;
	  change_flag=1;
	}
void dec_nachtabsenkung_bis_min()
	{
	  if(digit_minute ==0){digit_minute=60;}
	  digit_minute--;
	  print_digit(digit_minute);
	  nacht_bis_min=digit_minute;
	  change_flag=1;
	}

void inc_nachtabsenkung_licht()
	{
	  helligkeitsstufe++;
	  if(helligkeitsstufe >10){helligkeitsstufe=10;}
	  print_digit(helligkeitsstufe);
	  lcd_helligkeit();
	  change_flag=1;
	}
void dec_nachtabsenkung_licht()
	{
	  if(helligkeitsstufe <1){helligkeitsstufe=1;}
	  helligkeitsstufe--;
	  print_digit(helligkeitsstufe);
	  lcd_helligkeit();
	  change_flag=1;
	}

void inc_datumuhrzeit_uhrzeit_std()
	{
	  digit_stunde++;
	  if(digit_stunde >23){digit_stunde=0;}
	  print_digit(digit_stunde);
	  change_time_flag=1;
	}
void dec_datumuhrzeit_uhrzeit_std()
	{
	  if(digit_stunde ==0){digit_stunde=24;}
	  digit_stunde--;
	  print_digit(digit_stunde);
	  change_time_flag=1;
	}

void inc_datumuhrzeit_uhrzeit_min()
	{
	  digit_minute++;
	  if(digit_minute >59){digit_minute=0;}
	  print_digit(digit_minute);
	  change_time_flag=1;
	}
void dec_datumuhrzeit_uhrzeit_min()
	{
	  if(digit_minute ==0){digit_minute=60;}
	  digit_minute--;
	  print_digit(digit_minute);
	  change_time_flag=1;
	}

void inc_datumuhrzeit_uhrzeit_sek()
	{
	  digit_sekunde++;
	  if(digit_sekunde >59){digit_sekunde=0;}
	  print_digit(digit_sekunde);
	  change_time_flag=1;
	}

void dec_datumuhrzeit_uhrzeit_sek()
	{
	  if(digit_sekunde ==0){digit_sekunde=60;}
	  digit_sekunde--;
	  print_digit(digit_sekunde);
	  change_flag=1;
	}

void inc_datumuhrzeit_weekday()
	{
	 wochentag++;
	 if(wochentag >6){wochentag=0;}
	 print_wochentag();
	 change_flag=1;
	}


void dec_datumuhrzeit_weekday()
	{
	 if(wochentag ==0){wochentag=7;}
	 wochentag--;
	 print_wochentag();
	 change_flag=1;
	}

void inc_datumuhrzeit_tag()
	{
	 tag++;
	 if(tag >31){tag=1;}
	 print_digit(tag);
	 change_flag=1;
	}

void dec_datumuhrzeit_tag()
	{
	 if(tag ==1){tag=32;}
	 tag--;
	 print_digit(tag);
	 change_flag=1;
	}

void inc_datumuhrzeit_monat()
	{
	 monat++;
	 if(monat >12){monat=1;}
	 print_digit(monat);
	 change_flag=1;
	}

void dec_datumuhrzeit_monat()
	{
	 if(monat ==1){monat=13;}
	 monat--;
	 print_digit(monat);
	 change_flag=1;
	}

void inc_datumuhrzeit_jahr()
	{
	 jahr++;
	 if(jahr >99){jahr=0;}
	 print_digit(jahr);
	 change_flag=1;
	}

void dec_datumuhrzeit_jahr()
	{
	 if(jahr ==0){jahr=100;}
	 jahr--;
	 print_digit(jahr);
	 change_flag=1;
	}

void inc_datumuhrzeit_reset()
	{
	  set_disp_pos(6,63);		// Display-Position
	  inverse=0;
	  print_text(text_25);	    // print Text "ok"
	  _delay_ms(2000);
	  set_disp_pos(6,63);		// Display-Position
	  print_text(text_26);		// print Text "wait.."
	  system_reset();
	}

//----------------------------------------------------------
//  Men-Funktion: Funktion in Men-Seite aufrufen
//----------------------------------------------------------
void menue_temperaturen()
	{
	  menue_auswahl++;
	  if(menue_auswahl >3){menue_auswahl=1;}
	  switch(menue_auswahl)
	   {
	     case 1 : menue_temperaturen_sollwert();
		   break;
         case 2 : menue_temperaturen_komfort();
		   break;
         case 3 : menue_temperaturen_nacht();
		   break;
       }
	}
void menue_komfort_funktion()
	{
	  menue_auswahl++;
	  if(menue_auswahl >4){menue_auswahl=1;}
	  switch(menue_auswahl)
	   {
	     case 1 : menue_komfort_temp();
		   break;
         case 2 : menue_komfort_laenge();
		   break;
         case 3 : menue_komfort_verzoegerung();
		   break;
         case 4 : menue_komfort_messdauer();
		   break;
       }
	}
void menue_nachtabsenkung()
	{
	  menue_auswahl++;
	  if(menue_auswahl >5){menue_auswahl=1;}
	  switch(menue_auswahl)
	   {
	     case 1 : menue_nachtabsenkung_von_std();
		   break;
	     case 2 : menue_nachtabsenkung_von_min();
		   break;
         case 3 : menue_nachtabsenkung_bis_std();
		   break;
         case 4 : menue_nachtabsenkung_bis_min();
		   break;
		 case 5 : menue_nachtabsenkung_licht();
		   break;
       }
	}
void menue_datum_uhrzeit()
	{
	  menue_auswahl++;
	  if(menue_auswahl >8){menue_auswahl=1;}
	  switch(menue_auswahl)
	   {
	     case 1 : menue_datumuhrzeit_datum_tag();
		   break;
	     case 2 : menue_datumuhrzeit_datum_monat();
		   break;
	     case 3 : menue_datumuhrzeit_datum_jahr();
		   break;
         case 4 : menue_datumuhrzeit_uhrzeit_std();
		   break;
         case 5 : menue_datumuhrzeit_uhrzeit_min();
		   break;
         case 6 : menue_datumuhrzeit_uhrzeit_sek();
		   break;
         case 7 : menue_datumuhrzeit_tag();
		   break;
         case 8 : menue_datumuhrzeit_reset();
		   break;
       }
	}

//----------------------------------------------------------
//  blinkender Cursor
//----------------------------------------------------------
void cursor_flash()
	{
	 set_disp_pos(cursor_ypos,cursor_xpos);		// Cursor-Position
	 inverse=0;
	  switch(cursor_flashing)
	   {
	     case 1 : print_char(0);				// Cursor on
		   break;
         case 0 : print_char(' ');				// Cursor off
	   	   break;
	   }
	}

//----------------------------------------------------------
//  LCD-Helligkeit ndern
//----------------------------------------------------------
void lcd_helligkeit()
	{
	 uint8_t hell=0;
	 hell=helligkeitsstufe*25;						// maximalen Helligkeitsgrad berechnen
	 if (adc_wert > hell){OCR0A = 255- hell;}		// aus ADC-Wert den PWM-Wert fr Timer berechnen
	 else{OCR0A=255-adc_wert;}						// falls ADC-Wert > als max. Helligkeit, dann max Wert 
	 last_adc_wert=adc_wert;						// alten ADC-Wert berschreiben
	}
										
//----------------------------------------------------------
//  Genderte Men-Werte sichern
//----------------------------------------------------------
void write_parameter()
	{
	 write_eeprom();
	 write_rtc_date();								// Uhrzeit
	}

//----------------------------------------------------------
//  Genderte Werte ins EEPROM schreiben
//----------------------------------------------------------
void write_eeprom()
	{
	 eeprom_busy_wait();		// warte bis EEPROM bereit
	 eeprom_write_word(&eetemp_soll,temp_soll_eeprom);
	 eeprom_busy_wait();		
	 eeprom_write_word(&eetemp_komfort,temp_komfort);
	 eeprom_busy_wait();		
	 eeprom_write_word(&eetemp_nacht,temp_nacht);
	 eeprom_busy_wait();	
	 eeprom_write_byte(&eekomfort_temp_funktion,komfort_temp_funktion);	// Komfort-Funktion
	 eeprom_busy_wait();
	 eeprom_write_byte(&eenacht_von_std,nacht_von_std);
	 eeprom_busy_wait();
	 eeprom_write_byte(&eenacht_von_min,nacht_von_min);
	 eeprom_busy_wait();
	 eeprom_write_byte(&eenacht_bis_std,nacht_bis_std);
	 eeprom_busy_wait();
	 eeprom_write_byte(&eenacht_bis_min,nacht_bis_min);
	 eeprom_busy_wait();
	 eeprom_write_byte(&eelcd_helligkeit,helligkeitsstufe);	// Display-Helligkeit
	}
//----------------------------------------------------------
//  Werte aus EEPROM lesen
//----------------------------------------------------------
void read_eeprom()
	{
	 eeprom_busy_wait();		// warte bis EEPROM bereit
	 temp_soll=eeprom_read_word(&eetemp_soll);
	 temp_soll_eeprom=temp_soll;
	 eeprom_busy_wait();
	 temp_komfort=eeprom_read_word(&eetemp_komfort);
	 eeprom_busy_wait();
	 temp_nacht=eeprom_read_word(&eetemp_nacht);
	 eeprom_busy_wait();
	 komfort_temp_funktion=eeprom_read_byte(&eekomfort_temp_funktion);	// Komfort-Funktion
	 eeprom_busy_wait();
	 nacht_von_std=eeprom_read_byte(&eenacht_von_std);	// Nachtabsenkung
	 eeprom_busy_wait();
	 nacht_von_min=eeprom_read_byte(&eenacht_von_min);	// Nachtabsenkung
	 eeprom_busy_wait();
	 nacht_bis_std=eeprom_read_byte(&eenacht_bis_std);	// Nachtabsenkung
	 eeprom_busy_wait();
	 nacht_bis_min=eeprom_read_byte(&eenacht_bis_min);	// Nachtabsenkung
	 eeprom_busy_wait();
	 helligkeitsstufe=eeprom_read_byte(&eelcd_helligkeit);	// Display-Helligkeit
	}

//----------------------------------------------------------
// Tasten-Funktionen aufrufen
//----------------------------------------------------------
void key_left_funktion() // Menseiten auswhlen
	{
	 if(menue_funktion==0){inc_menue_site();menue_auswahl=0;}			// Menauswahl besttigen oder Regler auto/off schalten
	 if(change_flag==1){write_parameter(),change_flag=0;}				// gendert Men-Werte ins EEPROM schreiben 
	 if(change_time_flag==1){write_time(),change_time_flag=0;}			// genderte Uhrzeit in RTC schreiben
	}	 																
void key_right_funktion() // Men aktivieren/Temperaturregler off/auto
	{
	 if(menue_funktion==0 && menue_site==0){regler_switch();ds1820_stop_flag=0;} // Temperaturregler auto/off schalten
	 if(menue_funktion==0 && menue_site==1){menue_temperaturen();}		// Men "Temperaturen" aufrufen
	 if(menue_funktion==0 && menue_site==2){menue_komfort_funktion();}	// Men "Komfort-Funktion" aufrufen
	 if(menue_funktion==0 && menue_site==3){menue_nachtabsenkung();}	// Men "Nachtabsenkung" aufrufen
	 if(menue_funktion==0 && menue_site==4){menue_datum_uhrzeit();}		// Men "Datum/Uhrzeit" aufrufen
	}
void key_inc_funktion()	// Men-Werte inkrementieren
	{
	 if(menue_funktion==0 && menue_site==0){inc_temp_sollwert();}
	 if(menue_funktion==0 && menue_site==1 && menue_auswahl==1){inc_temperaturen_sollwert();}
	 if(menue_funktion==0 && menue_site==1 && menue_auswahl==2){inc_temperaturen_komfort();}
	 if(menue_funktion==0 && menue_site==1 && menue_auswahl==3){inc_temperaturen_nacht();}
	 if(menue_funktion==0 && menue_site==2 && menue_auswahl==1){komfortfunktion_komforttemp_on();}
	 if(menue_funktion==0 && menue_site==3 && menue_auswahl==1){inc_nachtabsenkung_von_std();}
	 if(menue_funktion==0 && menue_site==3 && menue_auswahl==2){inc_nachtabsenkung_von_min();}
	 if(menue_funktion==0 && menue_site==3 && menue_auswahl==3){inc_nachtabsenkung_bis_std();}
	 if(menue_funktion==0 && menue_site==3 && menue_auswahl==4){inc_nachtabsenkung_bis_min();}
	 if(menue_funktion==0 && menue_site==3 && menue_auswahl==5){inc_nachtabsenkung_licht();}
	 if(menue_funktion==0 && menue_site==4 && menue_auswahl==1){inc_datumuhrzeit_tag();}
	 if(menue_funktion==0 && menue_site==4 && menue_auswahl==2){inc_datumuhrzeit_monat();}
	 if(menue_funktion==0 && menue_site==4 && menue_auswahl==3){inc_datumuhrzeit_jahr();}
	 if(menue_funktion==0 && menue_site==4 && menue_auswahl==4){inc_datumuhrzeit_uhrzeit_std();}
	 if(menue_funktion==0 && menue_site==4 && menue_auswahl==5){inc_datumuhrzeit_uhrzeit_min();}
	 if(menue_funktion==0 && menue_site==4 && menue_auswahl==6){inc_datumuhrzeit_uhrzeit_sek();}
	 if(menue_funktion==0 && menue_site==4 && menue_auswahl==7){inc_datumuhrzeit_weekday();}
	 if(menue_funktion==0 && menue_site==4 && menue_auswahl==8){inc_datumuhrzeit_reset();}
	}
void key_dec_funktion() // Men-Werte dekrementieren
	{
	 if(menue_funktion==0 && menue_site==0){dec_temp_sollwert();}
	 if(menue_funktion==0 && menue_site==1 && menue_auswahl==1){dec_temperaturen_sollwert();}
	 if(menue_funktion==0 && menue_site==1 && menue_auswahl==2){dec_temperaturen_komfort();}
	 if(menue_funktion==0 && menue_site==1 && menue_auswahl==3){dec_temperaturen_nacht();}
	 if(menue_funktion==0 && menue_site==2 && menue_auswahl==1){komfortfunktion_komforttemp_off();}
	 if(menue_funktion==0 && menue_site==3 && menue_auswahl==1){dec_nachtabsenkung_von_std();}
	 if(menue_funktion==0 && menue_site==3 && menue_auswahl==2){dec_nachtabsenkung_von_min();}
	 if(menue_funktion==0 && menue_site==3 && menue_auswahl==3){dec_nachtabsenkung_bis_std();}
	 if(menue_funktion==0 && menue_site==3 && menue_auswahl==4){dec_nachtabsenkung_bis_min();}
	 if(menue_funktion==0 && menue_site==3 && menue_auswahl==5){dec_nachtabsenkung_licht();}
	 if(menue_funktion==0 && menue_site==4 && menue_auswahl==1){dec_datumuhrzeit_tag();}
	 if(menue_funktion==0 && menue_site==4 && menue_auswahl==2){dec_datumuhrzeit_monat();}
	 if(menue_funktion==0 && menue_site==4 && menue_auswahl==3){dec_datumuhrzeit_jahr();}
	 if(menue_funktion==0 && menue_site==4 && menue_auswahl==4){dec_datumuhrzeit_uhrzeit_std();}
	 if(menue_funktion==0 && menue_site==4 && menue_auswahl==5){dec_datumuhrzeit_uhrzeit_min();}
	 if(menue_funktion==0 && menue_site==4 && menue_auswahl==6){dec_datumuhrzeit_uhrzeit_sek();}
	 if(menue_funktion==0 && menue_site==4 && menue_auswahl==7){dec_datumuhrzeit_weekday();}
	 if(menue_funktion==0 && menue_site==4 && menue_auswahl==8){inc_datumuhrzeit_reset();}
	}

//----------------------------------------------------------
// Tasten-Abfrage
//----------------------------------------------------------
void read_keys()
	{
	 	// Taste "<" abfragen ------------
	 	if(key_left_status ==0 && !(PINB & (1<<PINB2)))
		{key_left_status=1;ds1820_stop_flag=1;_delay_ms(50);key_left_funktion();} // Taste wurde gedrckt springe zur Tastenfunktion
		else if(key_left_status ==1 && !(PINB & (1<<PINB2)))
		{key_left_status=2;_delay_ms(50);}								// Taste wird gehalten
		else if(key_left_status ==2 && (PINB & (1<<PINB2)))
		{key_left_status=3;_delay_ms(50);}								// Taste wird losgelassen
		else if(key_left_status ==3 && (PINB & (1<<PINB2)))
		{key_left_status=0;}											// Taste nicht gedrckt
		// Taste ">" abfragen ------------
	 	if(key_right_status ==0 && !(PINB & (1<<PINB3)))
		{key_right_status=1;ds1820_stop_flag=1;_delay_ms(50);key_right_funktion();}	// Taste wurde gedrckt springe zu Tasten-Funktion
		else if(key_right_status ==1 && !(PINB & (1<<PINB3)))
		{key_right_status=2;_delay_ms(50);}								// Taste wird gehalten
		else if(key_right_status ==2 && (PINB & (1<<PINB3)))
		{key_right_status=3;_delay_ms(50);}								// Taste wird losgelassen
		else if(key_right_status ==3 && (PINB & (1<<PINB3)))
		{key_right_status=0;}											// Taste nicht gedrckt
	 	// Taste "-" abfragen ------------
	 	if(key_minus_status ==0 && !(PINB & (1<<PINB0)))
		{key_minus_status=1;_delay_ms(10);key_dec_funktion();			// Taste wurde gedrckt
		 flash_big_ziffer_timer=0;										// blinkende Temperaturanzeige einschalten 
	     flash_big_ziffer=1;}											
		else if(key_minus_status ==1 && !(PINB & (1<<PINB0)))
		{key_minus_status=2;_delay_ms(500);}							// Taste wird gehalten
		else if(key_minus_status ==2 && (PINB & (1<<PINB0)))
		{key_minus_status=3;_delay_ms(50);}
		else if(key_minus_status ==3 && (PINB & (1<<PINB0)))
		{key_minus_status=0;_delay_ms(50);
		 flash_big_ziffer_timer=0;										// blinkende Temperaturanzeige einschalten 
	     flash_big_ziffer=1;}
		else if(key_minus_status ==2){key_dec_funktion();ds1820_stop_flag=1;_delay_ms(150);} // Tastenfunktion wiederholen	
	 	// Taste "+" abfragen ------------
	 	if(key_plus_status ==0 && !(PINB & (1<<PINB1)))					// Taste wurde gedrckt
		{key_plus_status=1;_delay_ms(10);key_inc_funktion();			
		 flash_big_ziffer_timer=0;										// blinkende Temperaturanzeige einschalten 
	     flash_big_ziffer=1;}			
		else if(key_plus_status ==1 && !(PINB & (1<<PINB1)))			// Taste wird gehalten
		{key_plus_status=2;_delay_ms(500);}								
		else if(key_plus_status ==2 && (PINB & (1<<PINB1)))
		{key_plus_status=3;_delay_ms(50);}								// Taste wird losgelassen
		else if(key_plus_status ==3 && (PINB & (1<<PINB1)))
		{key_plus_status=0;_delay_ms(50);
		 flash_big_ziffer_timer=0;										// blinkende Temperaturanzeige einschalten 
	     flash_big_ziffer=1;}
		else if(key_plus_status ==2){key_inc_funktion();ds1820_stop_flag=1;_delay_ms(150);}	// Tasten-Funktion wiederholen
		// Cursor-Blinken
		if(menue_timer_on==1 && cursor_on==1){cursor_flash();}			// Cursor-Blinken
		// Men-Timer
		if(menue_timer_on==1 && menue_timer >300)						// Men-Timer lnger als 5min
		 {																// dann Men-Timer stop,Cursor lschen
		   menue_timer_on=0;menue_timer=0;cursor_on=0;cursor_flash();	// Grundanzeige
		   print_grundanzeige();
		   ds1820_stop_flag=0;ds1820_busy_flag=1;
		   sensor_flag=1;menue_site=0;
		   if(regler==0){set_regler_off();}else{set_regler_auto();} 	
		 }
	}

//----------------------------------------------------------
// Temperaturwert anzeigen
//----------------------------------------------------------
void print_temperatur()
	{
	 uint16_t digit;
	 uint16_t temp_soll1a;
	 temp_soll1a=temp_soll1;
	 digit=temp_soll1a/1000;
	 if(digit >0){print_char(digit+48);}
	 else{print_char(' ');}
	 temp_soll1a=temp_soll1a%1000;
	 digit=temp_soll1a/100;
	 print_char(digit+48);
	 print_char('.');
	 temp_soll1a=temp_soll1a%100;
	 digit=temp_soll1a/10;
	 print_char(digit+48);
	}

//----------------------------------------------------------
//  Temperaturregelung: Nachtabsenkung 
//----------------------------------------------------------
void nachtabsenkung()
	{
	 nacht_von=((nacht_von_std*60)+nacht_von_min);	// Zeiten in min umrechen
	 nacht_bis=((nacht_bis_std*60)+nacht_bis_min);
	 nacht_time=((stunde*60)+minute);
   // aktuelle Uhrzeit mit Zeit fr Nachtabsenkung vergleichen
	 if((nacht_von<=nacht_time) && (nacht_bis>=nacht_time))
	    { 
	     if(nacht_flag==0)
		 {
		 temp_soll=temp_nacht;			// Nach-Temperatur aktivieren
		 temp_soll1=temp_nacht;			// Zwischenspeicher Temperaturanzeige
	     set_disp_pos(0,77);
		 inverse=1;
		 print_char('N');				// print "N" in Statusanzeige fr Nachtabsenkung
		 set_disp_pos(0,85);
		 print_temperatur();			// print Nacht-Temperatur in Statusanzeige
		 nacht_flag=1;					// Nacht-Flag setzen
		 }
	   } 
     else
	   {
	     if(nacht_flag==1)
		 {
		 temp_soll=temp_soll_eeprom;	// Soll-Temperatur aktivieren
		 temp_soll1=temp_soll_eeprom;
		 set_disp_pos(0,77);
		 inverse=1;
		 print_char(' ');				// "N" in Statusanzeige lschen
 		 set_disp_pos(0,85);
		 print_temperatur();			// print Solltemperatur in Statusanzeigen
		 nacht_flag=0;					// Nacht-Flag lschen
		 }
	   } 
	}

//-- Komfort-Funktion ausschalten -----------------------------------
void komfort_off()
	{
	  komfort_flag=0;			// Komfort-Funktion ausschalten
	  komfort_timer_on=0;
	  komfort_timer=0;
	  print_symbol(face2_sym,24,2);			// Face-Symbol lschen
	  set_disp_pos(0,70);		// "K"-Zeichen fr Komfort-Temperatur lschen
	  inverse =1;
	  print_char(' ');
	  temp_soll1=temp_soll;		// Soll-Temperatur anzeigen
	  set_disp_pos(0,85);		// Textposition
	  print_temperatur();
	}	 

//-- Temperaturregler ausschalten -----------------------------------
	void set_regler_off()
	{
	regler=0;					// Temperaturregelung = aus
	relais_off;					// Heizungs-Relais ausschalten
	heizen_flag=0;
	set_disp_pos(7,2);			// Temperaturregler-Modus anzeigen
	inverse=0;
	print_text(text_2);			// Text "off " anzeigen
	print_symbol(sonne3_sym,23,4); // Sonnen-Symbol lschen
	komfort_off();
	}

//-- Temeperaturregler auf Automatik schalten ------------------------
	void set_regler_auto()
	{	
	regler=1;					// Temperaturregelung = automatisch
	set_disp_pos(7,2);			// Temperaturregler-Modus anzeigen
	inverse=0;
	print_text(text_3);			// Text "aut." anzeigen
	if(komfort_temp_funktion==1)
	{komfort_timer_on=0;komfort_timer=0;pir_timer_on=0;pir_timer=0;komfort_flag=1;}
	}

// Temperatur-Komforttemperatur ---------------------
 void komfort_funktion()
	{
	  if(komfort_temp_funktion==1)
	  {
	  uint16_t komfort_time =600; 			// Komfort-Zeit = 10min
	  uint8_t pir_time1 =30;				// PIR-Zeit_1 = 30sec
	  uint8_t pir_time2 =60;				// PIR-Zeit_2 = 60sec
	  uint8_t face_time =3;					// Face-Symbol-Zeit = 3sec

		if(komfort_flag ==1)				// Komfort-Funktion eingeschaltet ?
		 {
		  if(komfort_timer_on ==1) 			// wenn ja, luft Komfort-Timer ?
		   {
		     if(komfort_timer >=komfort_time) 		// wenn ja, luft Komfort-Timer lnger als 900sec (15min) ?
			  {
			    komfort_timer_on =0;		// wenn ja, Komfort-Timer stoppen
				komfort_timer =0;			// und Komfort-Timer-Wert lschen
				set_disp_pos(0,70);			// "K"-Zeichen fr Komfort-Temperatur lschen
				inverse =1;
				print_char(' ');
				temp_soll1=temp_soll;
				inverse=1;
				set_disp_pos(0,85);
				print_temperatur();
			  } 
		   } 
		  if(pir_flag ==1)					// PIR-Signal empfangen ?
		   {
			pir_flag=0;						// wenn ja, PIR-Flag lschen
			if(face_timer >=face_time)		// Face-Timer prfen
			 {
			   face_timer =0;
			 }
			 else
			  {
			   face_timer_on=1;				// und Face-Symbol anzeigen
			   face_timer=0;
			   print_symbol(face1_sym,24,2);
			  }
			 if(komfort_timer_on ==1)		// Komfort-Timer-Wert lschen wenn PIR empfangen
			  {
				komfort_timer =0;			// Komfort-Timer-Wert auf 0
			  }
			if(pir_timer_on ==1)			// wenn ja, luft PIR-Timer ?
			 {
			   if(pir_timer >=pir_time2)	// wenn ja, luft PIR-Timer lnger als 60sec ?
			    {
				  pir_timer =0;				// wenn ja, PIR-Timer-Wert lschen
				}
			   else
			    if(pir_timer >=pir_time1)	// luft PIR-Timer lnger als 30sec ?
			    {
				  komfort_timer_on =1;		// wenn ja, Komfort-Timer starten
				  komfort_timer =0;			// Komfort-Timer-Wert lschen
				  set_disp_pos(0,70);		// "K"-Zeichen fr Komfort-Temperatur anzeigen
				  inverse =1;
				  print_char('K');
				  temp_soll1=temp_komfort;
				  inverse=1;
				  set_disp_pos(0,85);		// Textposition
				  print_temperatur();
		   		}
			 }
		    else
			{
			  pir_timer_on=1;
			}
		   }
		  else								// Kein PIR-Signal empgangen
		   {
			 if(pir_timer_on ==1)			// wenn ja, luft PIR-Timer ?
			  {
				if(pir_timer >=pir_time2)	// wenn ja, luft PIR-Timer lnger als 60sec ?
				 {
				   pir_timer_on =0;			// PIR-Timer stoppen
				   pir_timer =0;			// PIR-Timer-Wert lschen
				 }
			  }
			 if(face_timer_on ==1)			// Face-Symbol nach 3sec lschen
		   	  {
			    if(face_timer >=face_time)
			     {
				   face_timer_on =0;
				   face_timer =0;
				   print_symbol(face2_sym,24,2);
			     }
			  }
		   }
		 }
		}		
	}


//----------------------------------------------------------
// init. Analog-Digital Wandler (Display-Helligkeit)
//----------------------------------------------------------
void adc_init()
	{
      uint8_t i;
      uint16_t result;
      ADMUX = 0b00000001;                     	 		// Kanal 1 whlen (Port PF1)
      ADMUX |= (1<<REFS1) | (1<<REFS0); 			 	// interne Referenzspannung nutzen
      ADCSRA = (1<<ADEN) | (1<<ADPS1) | (1<<ADPS0);  	// Frequenzvorteiler setzen auf 8 (1) 
                               						 	// und ADC aktivieren (1)
   /* nach Aktivieren des ADC wird ein "Dummy-Readout" empfohlen, man liest
      also einen Wert und verwirft diesen, um den ADC "warmlaufen zu lassen" */
      ADCSRA |= (1<<ADSC);              				// eine ADC-Wandlung 
      while ( ADCSRA & (1<<ADSC) ) {;}     				// auf Abschluss der Konvertierung warten 
      result = ADCW;  									// ADCW muss einmal gelesen werden,
                  										// sonst wird Ergebnis der nchsten Wandlung
                  										// nicht bernommen.
   /* Eigentliche Messung - Mittelwert aus 4 aufeinanderfolgenden Wandlungen */
      result = 0; 
      for( i=0; i<4; i++ )
       {
         ADCSRA |= (1<<ADSC);            				// eine Wandlung "single conversion"
         while ( ADCSRA & (1<<ADSC) ) {;}   	    	// auf Abschluss der Konvertierung warten
         result += ADCW;		    					// Wandlungsergebnisse aufaddieren
       }
      ADCSRA &= ~(1<<ADEN);             				// ADC deaktivieren (2)
      result /= 4;                     					// Summe durch vier teilen = arithm. Mittelwert
      result/=2;
      if (result >249){result=249;}						// Wert 249 fr Grundhelligkeit
      adc_wert=255-result;
      lcd_helligkeit();
      if(adc_wert >=last_adc_wert+10){lcd_helligkeit();} // Hysteresenfunktion damit Beleuchtung nicht flackert
      if(adc_wert <=last_adc_wert-10){lcd_helligkeit();}
	}

//----------------------------------------------------------
//  System-Reset
//----------------------------------------------------------
void system_reset()
	{
	jahr=10;					// Datum und Uhrzeit setzen
	monat=1;
	tag=1;
	wochentag=0;
	stunde=0;
	minute=0;
	sekunde=0;
	jahr=((((jahr)/10)<<4)|((jahr)%10));	// Dezimal in BCD umwandeln
	monat=((((monat)/10)<<4)|((monat)%10));
	tag=((((tag)/10)<<4)|((tag)%10));
	stunde=((((stunde)/10)<<4)|((stunde)%10));
	minute=((((minute)/10)<<4)|((minute)%10));
	sekunde=((((sekunde)/10)<<4)|((sekunde)%10));
	i2c_start();
	i2c_write(0xA2);
	i2c_write(0x00);
	i2c_write(0);			// Reg. 0x00 	control_status_1
	i2c_write(0);			// Reg. 0x01 	control_status_2
	i2c_write(sekunde);		// Reg. 0x02 	VL_seconds
	i2c_write(minute);		// Reg. 0x03	minutes
	i2c_write(stunde);		// Reg. 0x04	hours
	i2c_write(tag);			// Reg. 0x05	days
	i2c_write(wochentag);	// Reg. 0x06	weekdays
	i2c_write(monat);		// Reg. 0x07	century_months
	i2c_write(jahr);		// Reg. 0x08	years
	i2c_write(0);
	i2c_write(0);
	i2c_write(0);
	i2c_write(0);
	i2c_write(0);
	i2c_write(0);
	i2c_write(0);
	i2c_stop();
	_delay_ms(2000);
	WDTCR |= (1<< WDCE);   	// Watchdog enabled
	WDTCR |= (1<< WDE);
	WDTCR = 0x00;			// Watchdog off
	}
//#################################################################
// Hauptprogramm
//#################################################################

int main (void)
     {
// init Power Reduction Register im Atmega zwecks Energieeinsparung
	PRR |= (1<<PRLCD);		// LCD-Controller disabled
	PRR |= (1<<PRUSART0);	// USART disabled
// init Relais
	relais_out;				// Port-Leitung auf Ausgang
	relais_off;				// Relais ausschalten
// init Portleitungen fr Display
	DDRA |= (1<<DDA0);  	// Ports auf Ausgang
	DDRA |= (1<<DDA1); 
	DDRA |= (1<<DDA2); 
	DDRA |= (1<<DDA3); 
	DDRD |= (1<<DDD7);
// init PWM-Portleitung fr LCD-Beleuchtung
	DDRB |= (1<<PB4);
// init Tasten-Port
	DDRB &= ~(1<<DDB2); 	// Portleitung PB2 (Taste-Links) als Eingang
	PORTB |= (1<<PB2); 		// Pullup einschalten
	DDRB &= ~(1<<DDB3); 	// Portleitung PB3 (Taste-Rechts) als Eingang
	PORTB |= (1<<PB3); 		// Pullup einschalten
	DDRB &= ~(1<<DDB0); 	// Portleitung PB2 (Taste-Plus) als Eingang
	PORTB |= (1<<PB0); 		// Pullup einschalten
	DDRB &= ~(1<<DDB1); 	// Portleitung PB2 (Taste-Minus) als Eingang
	PORTB |= (1<<PB1); 		// Pullup einschalten
// init PIR-Interrupt (Bewegungsmelder)
	DDRD &= ~(1<<DDD1); 	// Portleitung PB2 (Taste-Minus) als Eingang
	PORTD |= (1<<PD1); 		// Pullup einschalten
// init Portleitungen fr Temperatursensor
	sens_ddr |= (1<<sensor1);     // Temp1-Portleitung auf Ausgang konfigurieren
    sens_port |=(1<<sensor1);     // Temp1-Leitung auf 1 setzen
// init Display
	ResetDOG();
	_delay_ms(5);
	initDOGM128();
	read_eeprom();				// Standartwerte aus EEPROM laden
	print_grundanzeige();		// Grundanzeige darstellen
	print_big_ziffer(38,10);	// "--.-" Zeichen anzeigen
	print_big_ziffer(70,10);
	print_kleine_ziffer(10);
// init Temperatursensor (1.Messwert verwerfen)
	ds1820_start();				// Temperaturmessung starten 
	_delay_ms(500);				// fr neuen Messung warten	
	sensor_flag=1;				// Flag=1 fr sofortige Temperaturanzeige in Programmschleife
// init. I2C-Bus
	i2c_init();
// init RTC (RealTimeClock)
	init_rtc();
//konfiguriere externen Interrupt fr Bewegungsmelder
	EICRA	|= (1<<ISC01);		// negative Flanke INT0
	EIMSK |= (1<<INT0);			// ext. INT0 aktiv
// init. Timer_1
	TCCR1A =0;
	TCCR1B |= (1<<WGM12);		// CTC Modus
	TCCR1B |= (1 << CS11);		// Timer1 starten, Frequens = F/8
	OCR1A = 2500;				// Timer1 Compare-Wert fr 20ms
	TIMSK1 |= (1<<OCIE1A); 		// enable Timer1 overflow interrupt
// PWM-Signal (Timer0) fr LCD-Beleuchtung
    TCCR0A |= (1<<CS01);      	// Fast PWM-Mode 62.5KHz (CPU-Frq/256,kein Prescaler)
    TCCR0A |= (1<<WGM00);
	TCCR0A |= (1<<WGM01);
	TCCR0A |= (1<<COM0A1);
	TCCR0A |= (1<<COM0A0);
	OCR0A = 255;             	// Voreinstellung Display-Beleuchtung aus
// Globalen Interuppt zulassen      
     sei();
// Programmschleife
	while(1)
	{
	 if(menue_site==0)			// Wenn eine Menseite aufgerufen wurde, 
	 {							// dann springe zu Funktion "read_keys()" 
	   read_time();				// aktuelle Uhrzeit und Wochentag in Statuszeile anzeigen
	   temperatur_anzeigen();	// Raumtemperatur anzeigen
	   relais_test();			// Funktion fr Relaissteurung aufrufen
       komfort_funktion();		// Funktion fr Komforttemperatur aufrufen
	   nachtabsenkung(); 		// Nachtabsenkung prfen
	   adc_init();				// Funktion fr ADC-Wandler aufrufen
	 }
	 read_keys();
    }
	return 0;
   }





